#ifndef TDS3DListenerH
#define TDS3DListenerH

// ==========================================================================
// File:         TDS3DListener.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDS3DListener Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDS3DListener
// Description: The TDS3DListener component wraps the DirectX DS3DLISTENER 
//              structure which describes the 3D world properties and 
//              position for the listener. 
//              
//              This component is used when setting or retrieving all the 
//              properties of a 3D sound listener at once using 
//              TDx_Sound3DListener::GetAllParameters() and 
//              TDx_Sound3DListener::SetAllParameters(). 
//              
//              For 3D sound to be effective at least one 3D sound listener 
//              and one 3D sound buffer is required. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDS3DListener : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDS3DListener : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DS3DLISTENER) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDS3DListener component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    DistanceFactor
    // Description: The DistanceFactor property defines a D3DVALUE 
    //              representing the current distance factor for the 
    //              listener. 
    // ----------------------------------------------------------------------
    __property D3DVALUE DistanceFactor = { read=FGetDistanceFactor, 
    write=FSetDistanceFactor, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    DopplerFactor
    // Description: The DopplerFactor property defines a D3DVALUE 
    //              representing the current doppler shift factor for the 
    //              listener. 
    // ----------------------------------------------------------------------
    __property D3DVALUE DopplerFactor = { read=FGetDopplerFactor, 
    write=FSetDopplerFactor, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDS3DListener method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    OrientFront
    // Description: The OrientFront property defines a D3DVECTOR indicating 
    //              the listener's front orientation. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR OrientFront = { read=FGetOrientFront, 
    write=FSetOrientFront, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    OrientTop
    // Description: The OrientTop property defines a D3DVECTOR indicating the 
    //              listener's top orientation. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR OrientTop = { read=FGetOrientTop, 
    write=FSetOrientTop, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Position
    // Description: The Position property defines a D3DVECTOR which describes 
    //              the listener's position in 3D space. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR Position = { read=FGetPosition, write=FSetPosition, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    RolloffFactor
    // Description: The RolloffFactor property defines a D3DVALUE 
    //              representing the current rolloff factor for the listener. 
    // ----------------------------------------------------------------------
    __property D3DVALUE RolloffFactor = { read=FGetRolloffFactor, 
    write=FSetRolloffFactor, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Velocity
    // Description: The Velocity property defines a D3DVECTOR representing 
    //              the listener's velocity in 3D space. 
    //              The value of this property is independent of the actual 
    //              position and movement of a listener, the application is 
    //              responsible for setting the appropriate velocity value. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR Velocity = { read=FGetVelocity, write=FSetVelocity, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDS3DListener's internal DS3DLISTENER structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDS3DListener(TComponent* Owner); 
    virtual __fastcall ~TDS3DListener(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DS3DLISTENER* Internal_DS3DLISTENER_Ptr = { 
    read=FGetInternal_DS3DLISTENER_Ptr, nodefault }; 

    void __fastcall Internal_DS3DLISTENER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    D3DVALUE __fastcall FGetDistanceFactor(); 
    void __fastcall FSetDistanceFactor( D3DVALUE pDistanceFactor ); 
    D3DVALUE __fastcall FGetDopplerFactor(); 
    void __fastcall FSetDopplerFactor( D3DVALUE pDopplerFactor ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    D3DVECTOR __fastcall FGetOrientFront(); 
    void __fastcall FSetOrientFront( D3DVECTOR pOrientFront ); 
    D3DVECTOR __fastcall FGetOrientTop(); 
    void __fastcall FSetOrientTop( D3DVECTOR pOrientTop ); 
    D3DVECTOR __fastcall FGetPosition(); 
    void __fastcall FSetPosition( D3DVECTOR pPosition ); 
    D3DVALUE __fastcall FGetRolloffFactor(); 
    void __fastcall FSetRolloffFactor( D3DVALUE pRolloffFactor ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    D3DVECTOR __fastcall FGetVelocity(); 
    void __fastcall FSetVelocity( D3DVECTOR pVelocity ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DS3DLISTENER* __fastcall FGetInternal_DS3DLISTENER_Ptr(); 
    void __fastcall FSetInternal_DS3DLISTENER( DS3DLISTENER* pDS3DLISTENER ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DS3DLISTENER fDS3DLISTENER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
