#ifndef TDSBCapsH
#define TDSBCapsH

// ==========================================================================
// File:         TDSBCaps.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDSBCaps Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// ==========================================================================
// Set:         TDSBCapsFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDSBCaps::Flags property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the Flags property 
//              description below. 
// ==========================================================================
enum TDSBCaps_Flags_Flags
    {
    dsbcaps_ctrl3d,
    dsbcaps_ctrlfrequency_,
    dsbcaps_ctrlpan,
    dsbcaps_ctrlpositionnotify,
    dsbcaps_ctrlvolume_,
    dsbcaps_getcurrentposition2,
    dsbcaps_globalfocus_,
    dsbcaps_locdefer_,
    dsbcaps_lochardware_,
    dsbcaps_locsoftware,
    dsbcaps_mute3datmaxdistance,
    dsbcaps_primarybuffer,
    dsbcaps_static_,
    dsbcaps_stickyfocus_
    };
typedef Set< TDSBCaps_Flags_Flags, 
             dsbcaps_ctrl3d, 
             dsbcaps_stickyfocus_ >  TDSBCaps_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDSBCaps
// Description: The TDSBCaps component wraps the DirectX DSBCAPS structure 
//              which describes the capabilities of a sound buffer. 
//              
//              This component is used when retrieving the capabilities of a 
//              sound buffer using TDx_SoundBuffer::GetCaps(). 
//              
//              This component differs from the TDSBufferDesc component in 
//              that it contains additional information such as the buffers 
//              location (hardware or software memory) and some properties 
//              relating to performance (download to hardware time and CPU 
//              mixing overhead). 
//              
//              At least one notification event must be defined for buffers 
//              created with DSBCAPS_CTRLPOSITIONNOTIFY set, otherwise the 
//              behavior is undefined. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDSBCaps : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDSBCaps : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags specifying the 
    //              capabilities of the sound buffer. 
    //              The described effect applies when the flag is set. 
    //              
    //              For optimum performance only specify control options that 
    //              will be used. 
    //              Attempts to use a control that is not set on a buffer 
    //              will result in the method failing. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDSBCaps_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    BufferBytes
    // Description: The BufferBytes property defines the size, in bytes, of 
    //              the buffer. 
    // ----------------------------------------------------------------------
    __property dword BufferBytes = { read=FGetBufferBytes, 
    write=FSetBufferBytes, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PlayCpuOverhead
    // Description: The PlayCpuOverhead property represents the percentage of 
    //              main processing cycles needed to mix this sound buffer. 
    //              For a hardware buffer the value will be 0, for a software 
    //              buffer, the value will depend on the buffer's format and 
    //              the speed of the system processor. 
    // ----------------------------------------------------------------------
    __property dword PlayCpuOverhead = { read=FGetPlayCpuOverhead, 
    write=FSetPlayCpuOverhead, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DSBCAPS) }; 

    // ----------------------------------------------------------------------
    // Property:    UnlockTransferRate
    // Description: The UnlockTransferRate property describes the rate at 
    //              which data is transferred, in kilobytes per second, to 
    //              buffer memory when TDx_SoundBuffer::Unlock() is called. 
    //              
    //              This value, along with the number of bytes, can be used 
    //              by applications to determine the time required for the 
    //              Unlock to execute. 
    //              Software buffers located in system memory will have a 
    //              very high rate because no processing is required. 
    //              Hardware buffers may have a slower rate because the 
    //              buffer might have to be downloaded to the sound card. 
    // ----------------------------------------------------------------------
    __property dword UnlockTransferRate = { read=FGetUnlockTransferRate, 
    write=FSetUnlockTransferRate, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDSBCaps component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDSBCaps method or fget/fset. 
    //              eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags specifying the 
    //              capabilities of the sound buffer. 
    //              The described effect applies when the flag is set. 
    //              
    //              For optimum performance only specify control options that 
    //              will be used. 
    //              Attempts to use a control that is not set on a buffer 
    //              will result in the method failing. 
    // Flags:       DSBCAPS_CTRL3D -
    //                The buffer is a primary or 3D secondary buffer. 
    //                For this to be a primary buffer, DSBCAPS_PRIMARYBUFFER 
    //                should be set in TDSBufferDesc::Flags. 
    //              DSBCAPS_CTRLFREQUENCY -
    //                The buffer has frequency control capability. 
    //                Frequency Control control is the ability to modify the 
    //                frequency, in samples per second, at which the buffer 
    //                is playing. 
    //              DSBCAPS_CTRLPAN -
    //                The buffer has pan control capability. 
    //                Pan control is the ability to modify the relative 
    //                values of the left and right audio channels. 
    //              DSBCAPS_CTRLPOSITIONNOTIFY -
    //                The buffer has position notification capability. 
    //                
    //                Position notification is the ability to set triggers, 
    //                which activate specified events, on reaching certain 
    //                positions or conditions within a sound buffer. 
    //                
    //                Setting DSBCAPS_CTRLPOSITIONNOTIFY when the hardware 
    //                uses VxD drivers forces the buffer to be a software 
    //                buffer. 
    //                VxD drivers don't support the notifications. 
    //                
    //                WDM drivers do support notifications so the buffer may 
    //                be located in hardware memory if available. 
    //                
    //                Calling TDx_SoundBuffer::Play() with 
    //                DSBPLAY_LOCHARDWARE will fail if this flag is set. 
    //              DSBCAPS_CTRLVOLUME -
    //                The buffer has volume control capability. 
    //                Volume control is the ability to modify the volume of a 
    //                sound buffer. 
    //              DSBCAPS_GETCURRENTPOSITION2 -
    //                TDx_SoundBuffer::GetCurrentPosition() uses the new 
    //                behavior of the play cursor and thus return a much more 
    //                accurate position. 
    //                
    //                This flag should always be set unless compatability 
    //                with DirectX 1 is required. 
    //                This flag only affects emulated sound cards. 
    //              DSBCAPS_GLOBALFOCUS -
    //                The buffer is a global sound buffer. 
    //                Global sound buffers will continue to play even when 
    //                the focus is switched to another application, unless 
    //                that application is utilizing the DSSCL_EXCLUSIVE or 
    //                DSSCL_WRITEPRIMARY cooperative levels. 
    //              DSBCAPS_LOCDEFER -
    //                The location of the buffer in hardware or software 
    //                memory can be decided when the buffer is played. 
    //              DSBCAPS_LOCHARDWARE -
    //                The buffer is located in hardware memory and uses 
    //                hardware mixing. 
    //              DSBCAPS_LOCSOFTWARE -
    //                The buffer is located in software memory and uses 
    //                software mixing. 
    //              DSBCAPS_MUTE3DATMAXDISTANCE -
    //                The buffer will cease playing when its maximum distance 
    //                is exceeded. 
    //                This option prevents the wasting of processor time by 
    //                not calculating and mixing sounds that are effectively 
    //                inaudible. 
    //              DSBCAPS_PRIMARYBUFFER -
    //                The buffer is a primary sound buffer. 
    //              DSBCAPS_STATIC -
    //                The buffer will be used for static sound data. 
    //                A sound buffer without this flag set will be a 
    //                streaming sound buffer. 
    //              DSBCAPS_STICKYFOCUS -
    //                The buffer has a sticky focus. 
    //                Having a sticky focus means that an application will 
    //                continue to play any sticky focus buffers it has when 
    //                the user switches to another application, as long as 
    //                the new focus application doesnt utilize DirectSound. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDSBCaps's internal DSBCAPS structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDSBCaps(TComponent* Owner); 
    virtual __fastcall ~TDSBCaps(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DSBCAPS* Internal_DSBCAPS_Ptr = { 
    read=FGetInternal_DSBCAPS_Ptr, nodefault }; 

    void __fastcall Internal_DSBCAPS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDSBCaps_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDSBCaps_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetBufferBytes(); 
    void __fastcall FSetBufferBytes( dword pBufferBytes ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetPlayCpuOverhead(); 
    void __fastcall FSetPlayCpuOverhead( dword pPlayCpuOverhead ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetUnlockTransferRate(); 
    void __fastcall FSetUnlockTransferRate( dword pUnlockTransferRate ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DSBCAPS* __fastcall FGetInternal_DSBCAPS_Ptr(); 
    void __fastcall FSetInternal_DSBCAPS( DSBCAPS* pDSBCAPS ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DSBCAPS fDSBCAPS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
