#ifndef TDSBPositionNotifyH
#define TDSBPositionNotifyH

// ==========================================================================
// File:         TDSBPositionNotify.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDSBPositionNotify Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// --------------------------------------------------------------------------
// Definition for the OnNotify event.
typedef __fastcall void (__closure*     TDx_NotifyEvent) 
                        ( TObject*      Sender, 
                          int           pNotification ); 

// ==========================================================================
// Class:       TDSBPositionNotify
// Description: The TDSBPositionNotify component wraps multiple DirectX 
//              DSBPOSITIONNOTIFY structures which define where in a sound 
//              buffer to call a specified event. 
//              
//              This component is used by 
//              TDx_SoundNotify::SetNotificationPositions() when setting one 
//              or more points within a sound or capture buffer at which the 
//              TDSBPositionNotify::OnNotify() or TDx_SoundNotify::OnNotify() 
//              events will be called. 
//              
//              To setup this component, first set the 
//              TDSBPositionNotify::ArraySize property, then set each 
//              TDSBPositionNotify::Offset property using array indexes, eg: 
//              
//                  DSBPositionNotify1->ArraySize = 1; 
//                  DSBPositionNotify1->Offset[0] = buffer_size / 2; 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDSBPositionNotify : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDSBPositionNotify : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDSBPositionNotify component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnNotify()
    // Description: The OnNotify event is triggered when the buffer reaches a 
    //              point previously defined using 
    //              TDx_SoundNotify::SetNotificationPositions(). 
    //              This is useful for loading or writing the next chunk of 
    //              data, cascading sound effects and so on. 
    //              Set TDx_SoundNotify::LocalNotify to true if you wish to 
    //              use the TDx_SoundNotify::OnNotify() event or false to use 
    //              this notification event. 
    // ----------------------------------------------------------------------
    __property TDx_NotifyEvent OnNotify = {read=FOnNotify, write=FOnNotify, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              DSBPOSITIONNOTIFY structures wrapped inside the 
    //              TDSBPositionNotify component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TDSBPositionNotify component wraps a single 
    //              DSBPOSITIONNOTIFY structure. To wrap multiple structures, 
    //              set the TDSBPositionNotify::ArraySize to the total number 
    //              of internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDSBPositionNotify method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    EventNotify
    // Description: The EventNotify property defines the handle of the Win32 
    //              event to be signaled whenever the condition contained in 
    //              the TDSBPositionNotify::Offset is achieved. 
    // ----------------------------------------------------------------------
    __property HANDLE EventNotify[dword pArrayIndex] = { 
    read=FGetEventNotify, write=FSetEventNotify }; 

    // ----------------------------------------------------------------------
    // Property:    Offset
    // Description: The Offset property defines the position of a 
    //              notification event trigger. 
    //              
    //              This value is either an offset, in bytes, from the 
    //              beginning of the sound buffer or DSBPN_OFFSETSTOP. 
    //              
    //              DSBPN_OFFSETSTOP doesnt represent a specific point in the 
    //              buffer but triggers a notification event when the buffer 
    //              ceases playing or recording. 
    // ----------------------------------------------------------------------
    __property dword Offset[dword pArrayIndex] = { read=FGetOffset, 
    write=FSetOffset }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal DSBPOSITIONNOTIFY structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal DSBPOSITIONNOTIFY structures. This result is the 
    //              same as multiplying TDSBPositionNotify::ArraySize and 
    //              TDSBPositionNotify::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDSBPositionNotify's internal DSBPOSITIONNOTIFY 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TDSBPositionNotify::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal DSBPOSITIONNOTIFY structures of the 
    //              TDSBPositionNotify component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDSBPositionNotify(TComponent* Owner); 
    virtual __fastcall ~TDSBPositionNotify(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DSBPOSITIONNOTIFY* Internal_DSBPOSITIONNOTIFY_Ptr = { 
    read=FGetInternal_DSBPOSITIONNOTIFY_Ptr, nodefault }; 

    void __fastcall Internal_DSBPOSITIONNOTIFY_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    HANDLE __fastcall FGetEventNotify( dword pArrayIndex ); 
    void __fastcall FSetEventNotify( dword pArrayIndex, HANDLE pEventNotify 
    ); 
    dword __fastcall FGetOffset( dword pArrayIndex ); 
    void __fastcall FSetOffset( dword pArrayIndex, dword pOffset ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DSBPOSITIONNOTIFY* __fastcall FGetInternal_DSBPOSITIONNOTIFY_Ptr(); 
    void __fastcall FSetInternal_DSBPOSITIONNOTIFY( dword pArrayIndex, 
    DSBPOSITIONNOTIFY* pDSBPOSITIONNOTIFY ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DSBPOSITIONNOTIFY* fDSBPOSITIONNOTIFY; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;
    TDx_NotifyEvent FOnNotify;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
