#ifndef TDSCBufferDescH
#define TDSCBufferDescH

// ==========================================================================
// File:         TDSCBufferDesc.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDSCBufferDesc Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// ==========================================================================
// Set:         TDSCBufferDescFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDSCBufferDesc::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDSCBufferDesc_Flags_Flags
    {
    dscbcaps_wavemapped
    };
typedef Set< TDSCBufferDesc_Flags_Flags, 
             dscbcaps_wavemapped, 
             dscbcaps_wavemapped >  TDSCBufferDesc_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDSCBufferDesc
// Description: The TDSCBufferDesc component wraps the DirectX DSCBUFFERDESC 
//              structure which describes the characteristics of a capture 
//              buffer. 
//              This component is used when a sound capture buffer is to be 
//              created using TDx_SoundCapture::CreateCaptureBuffer(). 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDSCBufferDesc : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDSCBufferDesc : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines a flag indicating capture 
    //              buffer capabilities. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDSCBufferDesc_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    BufferBytes
    // Description: The BufferBytes property defines the size, in bytes, of 
    //              the new capture buffer. 
    // ----------------------------------------------------------------------
    __property dword BufferBytes = { read=FGetBufferBytes, 
    write=FSetBufferBytes, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DSCBUFFERDESC) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDSCBufferDesc component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDSCBufferDesc method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines a flag indicating capture 
    //              buffer capabilities. 
    //              The described effect applies when the flag is set. 
    // Flags:       DSCBCAPS_WAVEMAPPED -
    //                Unsupported sound formats will use the Win32 wave 
    //                mapper. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Format
    // Description: The Format property references a WAVEFORMATEX object 
    //              defining the waveform format of the new capture buffer. 
    // ----------------------------------------------------------------------
    __property WAVEFORMATEX* Format = { read=FGetFormat, write=FSetFormat, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDSCBufferDesc's internal DSCBUFFERDESC structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDSCBufferDesc(TComponent* Owner); 
    virtual __fastcall ~TDSCBufferDesc(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DSCBUFFERDESC* Internal_DSCBUFFERDESC_Ptr = { 
    read=FGetInternal_DSCBUFFERDESC_Ptr, nodefault }; 

    void __fastcall Internal_DSCBUFFERDESC_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDSCBufferDesc_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDSCBufferDesc_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetBufferBytes(); 
    void __fastcall FSetBufferBytes( dword pBufferBytes ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    WAVEFORMATEX* __fastcall FGetFormat(); 
    void __fastcall FSetFormat( WAVEFORMATEX* pFormat ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DSCBUFFERDESC* __fastcall FGetInternal_DSCBUFFERDESC_Ptr(); 
    void __fastcall FSetInternal_DSCBUFFERDESC( DSCBUFFERDESC* pDSCBUFFERDESC 
    ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DSCBUFFERDESC fDSCBUFFERDESC; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
