#ifndef TDSCCapsH
#define TDSCCapsH

// ==========================================================================
// File:         TDSCCaps.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDSCCaps Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// ==========================================================================
// Set:         TDSCCapsFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDSCCaps::Flags property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the Flags property 
//              description below. 
// ==========================================================================
enum TDSCCaps_Flags_Flags
    {
    dscaps_certified,
    dsccaps_emuldriver
    };
typedef Set< TDSCCaps_Flags_Flags, 
             dscaps_certified, 
             dsccaps_emuldriver >  TDSCCaps_Flags_Set;
// ==========================================================================
// Set:         TDSCCapsFormats_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDSCCaps::Formats property using the BCB Object Inspector. 
//              For detailed descriptions of each flag, see the Formats 
//              property description below. 
// ==========================================================================
enum TDSCCaps_Formats_Flags
    {
    wave_format_1m08,
    wave_format_1m16,
    wave_format_1s08,
    wave_format_1s16,
    wave_format_2m08,
    wave_format_2m16,
    wave_format_2s08,
    wave_format_2s16,
    wave_format_4m08,
    wave_format_4m16,
    wave_format_4s08,
    wave_format_4s16
    };
typedef Set< TDSCCaps_Formats_Flags, 
             wave_format_1m08, 
             wave_format_4s16 >  TDSCCaps_Formats_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDSCCaps
// Description: The TDSCCaps component wraps the DirectX DSCCAPS structure 
//              which describes the capture capabilities of a sound device. 
//              This component is used when retrieving the capabilities of a 
//              sound capture buffer using TDx_SoundCapture::GetCaps(). 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDSCCaps : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDSCCaps : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags specifying the available 
    //              hardware capture buffer capabilities. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDSCCaps_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Formats_
    // Description: The Formats property defines flags indicating the 
    //              standard wave formats that are supported. 
    //              These formats are the same as those used by the 
    //              WAVEINCAPS structure utilized in the Win32 waveform audio 
    //              functions. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Formats' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Formats 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDSCCaps_Formats_Set Formats_ = { read=FGetFormats_, 
    write=FSetFormats_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Channels
    // Description: The Channels property defines the number of channels the 
    //              device supports. 
    //              1 is mono, 2 is stereo, etc. 
    // ----------------------------------------------------------------------
    __property dword Channels = { read=FGetChannels, write=FSetChannels, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DSCCAPS) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDSCCaps component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDSCCaps method or fget/fset. 
    //              eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags specifying the available 
    //              hardware capture buffer capabilities. 
    //              The described effect applies when the flag is set. 
    // Flags:       DSCAPS_CERTIFIED -
    //                The wave demodulation driver has been tested and 
    //                certified by Microsoft. 
    //              DSCCAPS_EMULDRIVER -
    //                There is no DirectSoundCapture driver installed for 
    //                this device, so it is being emulated using the standard 
    //                Windows waveform-audio functions. 
    //                Expect some degree of performance degradation. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Formats
    // Description: The Formats property defines flags indicating the 
    //              standard wave formats that are supported. 
    //              These formats are the same as those used by the 
    //              WAVEINCAPS structure utilized in the Win32 waveform audio 
    //              functions. 
    // Flags:       WAVE_FORMAT_1M08 -
    //                Wave formats that are 11.025 kHz, mono and 8-bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_1M16 -
    //                Wave formats that are 11.025 kHz, mono and 16 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_1S08 -
    //                Wave formats that are 11.025 kHz, stereo and 8 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_1S16 -
    //                Wave formats that are 11.025 kHz, stereo and 16 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_2M08 -
    //                Wave formats that are 22.05 kHz, mono and 8 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_2M16 -
    //                Wave formats that are 22.05 kHz, mono and 16 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_2S08 -
    //                Wave formats that are 22.05 kHz, stereo and 8 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_2S16 -
    //                Wave formats that are 22.05 kHz, stereo and 16 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_4M08 -
    //                Wave formats that are 44.1 kHz, mono and 8 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_4M16 -
    //                Wave formats that are 44.1 kHz, mono and 16 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_4S08 -
    //                Wave formats that are 44.1 kHz, stereo and 8 bit are 
    //                supported by the hardware. 
    //              WAVE_FORMAT_4S16 -
    //                Wave formats that are 44.1 kHz, stereo and 16 bit are 
    //                supported by the hardware. 
    // ----------------------------------------------------------------------
    __property dword Formats = { read=FGetFormats, write=FSetFormats, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDSCCaps's internal DSCCAPS structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Property:    Formats_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Formats property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Formats_Strings = { read=FGetFormats_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDSCCaps(TComponent* Owner); 
    virtual __fastcall ~TDSCCaps(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DSCCAPS* Internal_DSCCAPS_Ptr = { 
    read=FGetInternal_DSCCAPS_Ptr, nodefault }; 

    void __fastcall Internal_DSCCAPS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDSCCaps_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDSCCaps_Flags_Set pFlags_ ); 
    TDSCCaps_Formats_Set __fastcall FGetFormats_(); 
    void __fastcall FSetFormats_( TDSCCaps_Formats_Set pFormats_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetChannels(); 
    void __fastcall FSetChannels( dword pChannels ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetFormats(); 
    void __fastcall FSetFormats( dword pFormats ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

    TStringList* __fastcall FGetFormats_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DSCCAPS* __fastcall FGetInternal_DSCCAPS_Ptr(); 
    void __fastcall FSetInternal_DSCCAPS( DSCCAPS* pDSCCAPS ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;
    TStringList* fFormats_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DSCCAPS fDSCCAPS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
