#ifndef TDx_Sound3DBufferH
#define TDx_Sound3DBufferH

// ==========================================================================
// File:         TDx_Sound3DBuffer.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDx_Sound3DBuffer Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_Sound3DBuffer component.
class TDx_SoundBuffer;
// --------------------------------------------------------------------------
// external classes used by TDx_Sound3DBuffer methods.
class TDS3DBuffer;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_Sound3DBuffer
// Description: The TDx_Sound3DBuffer component wraps the DirectX 
//              IDirectSound3DBuffer interface, used to retrieve and set the 
//              position, orientation and environmental properties for a 
//              sound buffer within 3D space. 
//              
//              It can set or retrieve all the buffer's properties at once or 
//              individually as required, 
//              define absolute or relative 3D processing mode and the 
//              position or velocity of the buffer. 
//              
//              Other properties worked with involve maximum and minimum 
//              distances, the angles and orientation of sound cones, and the 
//              cone outside volume. 
//              
//              You may set or retrieve all a 3D buffers properties at once 
//              or individually as required. 
//              The application of changes can be delayed until 
//              TDx_Sound3DListener::CommitDefferedSettings() is called by 
//              setting DS3D_DEFERRED when calling methods that set new 
//              values for 3D buffer properties. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_Sound3DBuffer : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_Sound3DBuffer : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_Sound3DBuffer::Create() method after it has 
    //              successfully created the internal LPDIRECTSOUND3DBUFFER 
    //              used within the TDx_Sound3DBuffer component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTSOUND3DBUFFER 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_Sound3DBuffer 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_Sound3DBuffer::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_Sound3DBuffer::Destroy() method, just before the 
    //              TDx_Sound3DBuffer internal LPDIRECTSOUND3DBUFFER 
    //              interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_Sound3DBuffer::OnCreate() 
    //              event. For example, assume you have already setup the 
    //              TDx_Sound3DBuffer::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_Sound3DBuffer::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_Sound3DBuffer component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTSOUND3DBUFFER used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTSOUND3DBUFFER, call the 
    //              TDx_Sound3DBuffer::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_Sound3DBuffer method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTSOUND3DBUFFER interface used in the 
    //              TDx_Sound3DBuffer component and must be called before any 
    //              methods of the TDx_Sound3DBuffer component will function. 
    // Params:      pSoundBuffer - 
    //                The SoundBuffer parameter references the existing 
    //                TDx_SoundBuffer component for which the TDx_Sound3DBuffer 
    //                is to be created. The sound buffer referenced by this 
    //                parameter must have been created with the DSBCAPS_CTRL3D 
    //                flag set. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_SoundBuffer* pSoundBuffer ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTSOUND3DBUFFER interface used in the 
    //              TDx_Sound3DBuffer component and should be called when the 
    //              internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetAllParameters()
    // Description: The TDx_Sound3DBuffer::GetAllParameters method will 
    //              return the current 3D characteristics of the sound 
    //              buffer. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pBuffer3D - 
    //                The Buffer3D parameter references a TDS3DBuffer component 
    //                for holding the current 3D characteristics of the sound 
    //                buffer if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetAllParameters( TDS3DBuffer* pBuffer3D ); 

    // ----------------------------------------------------------------------
    // Method:      GetConeAngles()
    // Description: The TDx_Sound3DBuffer::GetConeAngles method will return 
    //              the inside and outside sound projection cone angles of 
    //              the sound buffer. 
    //              
    //              The minimum, maximum and default cone angles are defined 
    //              in the dsound.h DS3D_MINCONEANGLE, DS3D_MAXCONEANGLE and 
    //              DS3D_DEFAULTCONEANGLE members as 0, 360 and 360 degrees 
    //              respectively. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pInsideConeAngle - 
    //                The InsideConeAngle parameter will reference the inside 
    //                sound projection cone angle, in degrees, if this method 
    //                returns successsfully. 
    //              pOutsideConeAngle - 
    //                The OutsideConeAngle parameter will reference the outside 
    //                sound projection cone angle, in degrees, if this method 
    //                returns successsfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetConeAngles( dword* pInsideConeAngle, dword* 
    pOutsideConeAngle ); 

    // ----------------------------------------------------------------------
    // Method:      GetConeOrientation()
    // Description: The TDx_Sound3DBuffer::GetConeOrientation method will 
    //              return the sound projection cone orientation of the sound 
    //              buffer. 
    //              
    //              Orientation vectors are adjusted so as to have a 
    //              magnitude of 1 or less. Thus the value retrieved may not 
    //              be the same as the value set. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pOrientation - 
    //                The Orientation parameter references a D3DVECTOR for 
    //                holding the current sound projection cone orientation if 
    //                this method returns successfully. 
    //                The vector represents the center of the sound cone. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetConeOrientation( D3DVECTOR* pOrientation ); 

    // ----------------------------------------------------------------------
    // Method:      GetConeOutsideVolume()
    // Description: The TDx_Sound3DBuffer::GetConeOutsideVolume method will 
    //              return the current volume, in 100ths of a decibel, set 
    //              for the outside cone of the sound buffer. 
    //              
    //              This volume is expressed as an attenuation value and is 
    //              subtracted from the normal volume levels. 
    //              
    //              The minimum, maximum and default cone outside volumes are 
    //              defined in the dsound.h DSBVOLUME_MIN, DSBVOLUME_MAX and 
    //              DS3D_DEFAULTCONEOUTSIDEVOLUME members as -10000, 0 and 0 
    //              respectively. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pConeOutsideVolume - 
    //                The ConeOutsideVolume parameter will reference the 
    //                current volume setting for the outside cone of the buffer 
    //                if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetConeOutsideVolume( long* pConeOutsideVolume ); 

    // ----------------------------------------------------------------------
    // Method:      GetMaxDistance()
    // Description: The TDx_Sound3DBuffer::GetMaxDistance method will return 
    //              the maximum distance, in distance units, currently set 
    //              for the sound buffer. 
    //              
    //              The default maximum distance is defined in the dsound.h 
    //              DS3D_DEFAULTMAXDISTANCE member as 1000000000.0 and as 
    //              such can be regarded as effectively infinite. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pMaxDistance - 
    //                The MaxDistance parameter will reference the maximum 
    //                distance currently set for the buffer if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetMaxDistance( D3DVALUE* pMaxDistance ); 

    // ----------------------------------------------------------------------
    // Method:      GetMinDistance()
    // Description: The TDx_Sound3DBuffer::GetMinDistance method will return 
    //              the minimum distance, in distance units, currently set 
    //              for the sound buffer. 
    //              
    //              The default minimum distance is defined in the dsound.h 
    //              DS3D_DEFAULTMINDISTANCE member as 1.0. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pMinDistance - 
    //                The MinDistance parameter will reference the current 
    //                minimum distance set for the sound buffer if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetMinDistance( D3DVALUE* pMinDistance ); 

    // ----------------------------------------------------------------------
    // Method:      GetMode()
    // Description: The TDx_Sound3DBuffer::GetMode method will return the 
    //              current mode of 3D sound processing operation. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pMode - 
    //                The Mode parameter will reference flags indicating the 
    //                current mode setting if this method returns successfully. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3DMODE_DISABLE -
    //                    There is no 3D sound processing. 
    //                    Sounds will appear to originate from the center of 
    //                    the listeners head. 
    //                  DS3DMODE_HEADRELATIVE -
    //                    The 3D sound properties are relative to the 
    //                    properties of the listener. 
    //                    Absolute sound properties will be updated 
    //                    automatically as the listener's properties change, 
    //                    maintaining constant relative propeties as a 
    //                    result. 
    //                  DS3DMODE_NORMAL -
    //                    The 3D sounds are processed normally. 
    //                    This is the default. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetMode( dword* pMode ); 

    // ----------------------------------------------------------------------
    // Method:      GetPosition()
    // Description: The TDx_Sound3DBuffer::GetPosition method will retrieve 
    //              the current position, in distance units, of this sound 
    //              buffer in 3D space. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pPosition - 
    //                The Position parameter references a D3DVECTOR for holding 
    //                the current position of the sound buffer if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetPosition( D3DVECTOR* pPosition ); 

    // ----------------------------------------------------------------------
    // Method:      GetVelocity()
    // Description: The TDx_Sound3DBuffer::GetVelocity method will return the 
    //              velocity, in distance units per second, currently set for 
    //              this sound buffer. 
    //              
    //              This value is used to calculate doppler effects only, it 
    //              is up to the application to move the buffer 
    //              appropriately. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pVelocity - 
    //                The Velocity parameter references a D3DVECTOR for holding 
    //                the current velocity of the sound buffer if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVelocity( D3DVECTOR* pVelocity ); 

    // ----------------------------------------------------------------------
    // Method:      SetAllParameters()
    // Description: The TDx_Sound3DBuffer::SetAllParameters method will set 
    //              all the properties of the sound buffer to the specified 
    //              values. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pBuffer - 
    //                The Buffer parameter references a TDS3DBuffer component 
    //                holding information which describes the 3D 
    //                characteristics that are to be applied to this sound 
    //                buffer. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                settings should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetAllParameters( TDS3DBuffer* pBuffer, dword 
    pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetConeAngles()
    // Description: The TDx_Sound3DBuffer::SetConeAngles method will set the 
    //              inside and outside sound projection cone angles of the 
    //              sound buffer. 
    //              
    //              The minimum, maximum and default cone angles are defined 
    //              in the dsound.h DS3D_MINCONEANGLE, DS3D_MAXCONEANGLE and 
    //              DS3D_DEFAULTCONEANGLE members as 0, 360 and 360 degrees 
    //              respectively. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pInsideConeAngle - 
    //                The InsideConeAngle parameter defines the angle, in 
    //                degrees, to which the inside sound projection cone is to 
    //                be set. 
    //              pOutsideConeAngle - 
    //                The OutsideConeAngle parameter defines the angle, in 
    //                degrees, to which the outside sound projection cone is to 
    //                be set. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetConeAngles( dword pInsideConeAngle, dword 
    pOutsideConeAngle, dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetConeOrientation()
    // Description: The TDx_Sound3DBuffer::SetConeOrientation method will set 
    //              the sound projection cone orientation for this sound 
    //              buffer. 
    //              
    //              Cone angles and volume factors must also have been set 
    //              for this method to have any effect. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pX - 
    //                The X parameter defines a D3DVALUE representing the new X 
    //                coordinate of the sound cone orientation vector. 
    //              pY - 
    //                The Y parameter defines a D3DVALUE representing the new Y 
    //                coordinate of the sound cone orientation vector. 
    //              pZ - 
    //                The Z parameter defines a D3DVALUE representing the new Z 
    //                coordinate of the sound cone orientation vector. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetConeOrientation( D3DVALUE pX, D3DVALUE pY, 
    D3DVALUE pZ, dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetConeOutsideVolume()
    // Description: The TDx_Sound3DBuffer::SetConeOutsideVolume method will 
    //              set the volume, in 100ths of a decibel, for the outside 
    //              cone of the sound buffer. 
    //              
    //              This volume is expressed as an attenuation value and is 
    //              subtracted from the normal volume levels. 
    //              
    //              The minimum, maximum and default cone outside volumes are 
    //              defined in the dsound.h DSBVOLUME_MIN, DSBVOLUME_MAX and 
    //              DS3D_DEFAULTCONEOUTSIDEVOLUME members as -10000, 0 and 0 
    //              respectively. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pConeOutsideVolume - 
    //                The ConeOutsideVolume parameter defines the volume, in 
    //                100ths of a decibel, to which the outside cone of the 
    //                sound buffer is to be set. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetConeOutsideVolume( long pConeOutsideVolume, 
    dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetMaxDistance()
    // Description: The TDx_Sound3DBuffer::SetMaxDistance method will set the 
    //              maximum distance, in distance units, for the sound 
    //              buffer. 
    //              
    //              The default maximum distance is defined in the dsound.h 
    //              DS3D_DEFAULTMAXDISTANCE member as 1000000000.0 and as 
    //              such can be regarded as effectively infinite. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pMaxDistance - 
    //                The MaxDistance parameter defines the maximum distance 
    //                value to be set for the sound buffer. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetMaxDistance( D3DVALUE pMaxDistance, dword 
    pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetMinDistance()
    // Description: The TDx_Sound3DBuffer::SetMinDistance method will set the 
    //              minimum distance, in distance units, for the sound 
    //              buffer. 
    //              
    //              The default minimum distance is defined in the dsound.h 
    //              DS3D_DEFAULTMINDISTANCE member as 1.0. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pMinDistance - 
    //                The MinDistance parameter defines the minimum distance 
    //                value to be set for the sound buffer. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetMinDistance( D3DVALUE pMinDistance, dword 
    pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetMode()
    // Description: The TDx_Sound3DBuffer::SetMode method will set the mode 
    //              of 3D sound processing operations for this buffer. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pMode - 
    //                The Mode parameter defines flags indicating the 3D 
    //                processing mode to be set for the sound buffer. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3DMODE_DISABLE -
    //                    There is to be no 3D sound processing. 
    //                    Sounds will appear to originate from the center of 
    //                    the users head. 
    //                  DS3DMODE_HEADRELATIVE -
    //                    The 3D sound properties are to be relative to the 
    //                    properties of the listener. 
    //                    Absolute sound properties will be updated 
    //                    automatically as the listener's properties change, 
    //                    maintaining constant relative propeties as a 
    //                    result. 
    //                  DS3DMODE_NORMAL -
    //                    3D sounds are to be processed normally. 
    //                    This is the default setting. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetMode( dword pMode, dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetPosition()
    // Description: The TDx_Sound3DBuffer::SetPosition method will set the 
    //              position, in distance units, of this sound buffer in 3D 
    //              space. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pX - 
    //                The X parameter defines a D3DVALUE representing the new X 
    //                coordinate of the sound buffer in 3D space. 
    //                This value may be adjusted to prevent floating point 
    //                overflow. 
    //              pY - 
    //                The Y parameter defines a D3DVALUE representing the new Y 
    //                coordinate of the sound buffer in 3D space. 
    //                This value may be adjusted to prevent floating point 
    //                overflow. 
    //              pZ - 
    //                The Z parameter defines a D3DVALUE representing the new Z 
    //                coordinate of the sound buffer in 3D space. 
    //                This value may be adjusted to prevent floating point 
    //                overflow. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetPosition( D3DVALUE pX, D3DVALUE pY, D3DVALUE 
    pZ, dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetVelocity()
    // Description: The TDx_Sound3DBuffer::SetVelocity method will set the 
    //              velocity, in distance units per second, for the sound 
    //              buffer. 
    //              
    //              This value is only used to calculate doppler effects, it 
    //              is up to the application to move the buffer 
    //              appropriately. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pX - 
    //                The X parameter defines a D3DVALUE representing the new X 
    //                coordinate of the velocity vector for the sound buffer. 
    //                This value may be adjusted to prevent floating-point 
    //                overflow. 
    //              pY - 
    //                The Y parameter defines a D3DVALUE representing the new Y 
    //                coordinate of the velocity vector for the sound buffer. 
    //                This value may be adjusted to prevent floating-point 
    //                overflow. 
    //              pZ - 
    //                The Z parameter defines a D3DVALUE representing the new Z 
    //                coordinate of the velocity vector for the sound buffer. 
    //                This value may be adjusted to prevent floating-point 
    //                overflow. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied immediately. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until 
    //                    TDx_Sound3DListener::CommitDeferredSettings() is 
    //                    called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetVelocity( D3DVALUE pX, D3DVALUE pY, D3DVALUE 
    pZ, dword pApply ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_Sound3DBuffer(TComponent* Owner); 
    virtual __fastcall ~TDx_Sound3DBuffer(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Sound_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTSOUND3DBUFFER Internal_LPDIRECTSOUND3DBUFFER = { 
    read=FGetInternal_LPDIRECTSOUND3DBUFFER, 
    write=FSetInternal_LPDIRECTSOUND3DBUFFER, nodefault }; 
    __property LPDIRECTSOUND3DBUFFER* Internal_LPDIRECTSOUND3DBUFFER_Ptr = { 
    read=FGetInternal_LPDIRECTSOUND3DBUFFER_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTSOUND3DBUFFER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTSOUND3DBUFFER __fastcall FGetInternal_LPDIRECTSOUND3DBUFFER(); 
    void __fastcall FSetInternal_LPDIRECTSOUND3DBUFFER( LPDIRECTSOUND3DBUFFER 
    pLPDIRECTSOUND3DBUFFER ); 
    LPDIRECTSOUND3DBUFFER* __fastcall 
    FGetInternal_LPDIRECTSOUND3DBUFFER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTSOUND3DBUFFER fLPDIRECTSOUND3DBUFFER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
