#ifndef TDx_Sound3DListenerH
#define TDx_Sound3DListenerH

// ==========================================================================
// File:         TDx_Sound3DListener.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDx_Sound3DListener Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_Sound3DListener component.
class TDx_SoundBuffer;
// --------------------------------------------------------------------------
// external classes used by TDx_Sound3DListener methods.
class TDS3DListener;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_Sound3DListener
// Description: The TDx_Sound3DListener component wraps the DirectX 
//              IDirectSound3DListener interface which is used in the 
//              retrieval and setting of position, orientation and 
//              environmental properties for a listener within 3D space. 
//              
//              It includes defining the distance units, doppler shift and 
//              rolloff values used, as well as the position, orientation and 
//              velocity of the listener within 3D space. 
//              
//              You may set or retrieve all a listener's properties at once 
//              or individually as required. 
//              The application of changes can be delayed until 
//              TDx_Sound3DListener::CommitDefferedSettings() is called by 
//              setting DS3D_DEFERRED when calling methods that set new 
//              values for listener properties. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_Sound3DListener : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_Sound3DListener : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_Sound3DListener::Create() method after it has 
    //              successfully created the internal LPDIRECTSOUND3DLISTENER 
    //              used within the TDx_Sound3DListener component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTSOUND3DLISTENER 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_Sound3DListener 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_Sound3DListener::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_Sound3DListener::Destroy() method, just before the 
    //              TDx_Sound3DListener internal LPDIRECTSOUND3DLISTENER 
    //              interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_Sound3DListener::OnCreate() event. For example, 
    //              assume you have already setup the 
    //              TDx_Sound3DListener::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_Sound3DListener::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_Sound3DListener component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTSOUND3DLISTENER used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTSOUND3DLISTENER, call the 
    //              TDx_Sound3DListener::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_Sound3DListener method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Method:      CommitDeferredSettings()
    // Description: The TDx_Sound3DListener::CommitDeferredSettings method 
    //              will commit any deferred settings that have been applied 
    //              since the last call to this method. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CommitDeferredSettings(); 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTSOUND3DLISTENER interface used in the 
    //              TDx_Sound3DListener component and must be called before 
    //              any methods of the TDx_Sound3DListener component will 
    //              function. 
    // Params:      pSoundBuffer - 
    //                The SoundBuffer parameter references the existing 
    //                TDx_SoundBuffer component from which the 
    //                TDx_Sound3DListener is to be created. The sound buffer 
    //                referenced by this parameter must have been created with 
    //                the DSBCAPS_CTRL3D and DSBCAPS_PRIMARYBUFFER flag set. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_SoundBuffer* pSoundBuffer ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTSOUND3DLISTENER interface used in the 
    //              TDx_Sound3DListener component and should be called when 
    //              the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetAllParameters()
    // Description: The TDx_Sound3DListener::GetAllParameters method will 
    //              return the current 3D characteristics of the 3D world and 
    //              the listener. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pListener - 
    //                The Listener parameter references a TDS3DListener 
    //                component for holding the current state of the 3D world 
    //                and listener if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetAllParameters( TDS3DListener* pListener ); 

    // ----------------------------------------------------------------------
    // Method:      GetDistanceFactor()
    // Description: The TDx_Sound3DListener::GetDistanceFactor method will 
    //              return the current distance factor setting for the 
    //              listener. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pDistanceFactor - 
    //                The DistanceFactor parameter references a D3DVALUE for 
    //                holding the current distance factor setting for the 
    //                listener if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetDistanceFactor( D3DVALUE* pDistanceFactor ); 

    // ----------------------------------------------------------------------
    // Method:      GetDopplerFactor()
    // Description: The TDx_Sound3DListener::GetDopplerFactor method will 
    //              return the doppler factor currently set for the listener. 
    //              The minimum, maximum and default doppler values are 
    //              defined in the dsound.h DS3D_MINDOPPLERFACTOR, 
    //              DS3D_MAXDOPPLERFACTOR and DS3D_DEFAULTDOPPLERFACTOR 
    //              members as 0.0, 10.0 and 1.0 respectively. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pDopplerFactor - 
    //                The DopplerFactor parameter references a D3DVALUE for 
    //                holding the doppler factor currently set for the listener 
    //                if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetDopplerFactor( D3DVALUE* pDopplerFactor ); 

    // ----------------------------------------------------------------------
    // Method:      GetOrientation()
    // Description: The TDx_Sound3DListener::GetOrientation method will 
    //              return the orientation currently set for the listener. 
    //              The front and top vectors meet at right angles within the 
    //              listeners head. The vectors may be adjusted to ensure 
    //              they are at right angles with a magnitude of less than 
    //              one and thus the values of the vectors may not be the 
    //              same as when they were set. 
    //              The front vector indicates the direction of the listeners 
    //              nose, while the top vector points out the top of the 
    //              listeners head. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pOrientFront - 
    //                The OrientFront parameter references a D3DVECTOR for 
    //                holding the listener's front orientation if this method 
    //                returns successfully. 
    //              pOrientTop - 
    //                The OrientTop parameter references a D3DVECTOR for 
    //                holding this listener's top orientation if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetOrientation( D3DVECTOR* pOrientFront, 
    D3DVECTOR* pOrientTop ); 

    // ----------------------------------------------------------------------
    // Method:      GetPosition()
    // Description: The TDx_Sound3DListener::GetPosition method will return 
    //              the current position, in distance units, of the listener. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pPosition - 
    //                The Position parameter references a D3DVECTOR for holding 
    //                the listener's current position vector if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetPosition( D3DVECTOR* pPosition ); 

    // ----------------------------------------------------------------------
    // Method:      GetRolloffFactor()
    // Description: The TDx_Sound3DListener::GetRolloffFactor method will 
    //              return the rolloff factor currently set for the listener. 
    //              The minimum, maximum and default rolloff values are 
    //              defined in the dsound.h DS3D_MINROLLOFFFACTOR, 
    //              DS3D_MAXROLLOFFFACTOR and DS3D_DEFAULTROLLOFFFACTOR 
    //              members as 0.0, 10.0 and 1.0 respectively. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pRolloffFactor - 
    //                The RolloffFactor parameter references a D3DVALUE for 
    //                holding the current rolloff factor if this method returns 
    //                successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetRolloffFactor( D3DVALUE* pRolloffFactor ); 

    // ----------------------------------------------------------------------
    // Method:      GetVelocity()
    // Description: The TDx_Sound3DListener::GetVelocity method will return 
    //              the current velocity set for the listener. 
    //              This value is used to calculate doppler effects only, it 
    //              is up to the application to move the buffer 
    //              appropriately. 
    //              The default velocity is (0,0,0). 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pVelocity - 
    //                The Velocity parameter references a D3DVECTOR for holding 
    //                the listener's current velocity if this method returns 
    //                successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVelocity( D3DVECTOR* pVelocity ); 

    // ----------------------------------------------------------------------
    // Method:      SetAllParameters()
    // Description: The TDx_Sound3DListener::SetAllParameters method will set 
    //              the 3D characteristics for the 3D world and the listener. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              TDX_BADPARAMS 
    //              DSERR_INVALIDPARAM 
    // Params:      pListener - 
    //                The Listener parameter references a TDS3DListener 
    //                component holding the 3D listener properties to be set. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetAllParameters( TDS3DListener* pListener, dword 
    pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetDistanceFactor()
    // Description: The TDx_Sound3DListener::SetDistanceFactor method will 
    //              set the distance factor for the listener. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pDistanceFactor - 
    //                The DistanceFactor parameter defines the distance factor 
    //                to be set for the listener. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetDistanceFactor( D3DVALUE pDistanceFactor, 
    dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetDopplerFactor()
    // Description: The TDx_Sound3DListener::SetDopplerFactor method will set 
    //              the doppler factor for the listener. 
    //              The minimum, maximum and default doppler values are 
    //              defined in the dsound.h DS3D_MINDOPPLERFACTOR, 
    //              DS3D_MAXDOPPLERFACTOR and DS3D_DEFAULTDOPPLERFACTOR 
    //              members as 0.0, 10.0 and 1.0 respectively. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pDopplerFactor - 
    //                The DopplerFactor parameter defines the doppler factor to 
    //                be set for the listener. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetDopplerFactor( D3DVALUE pDopplerFactor, dword 
    pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetOrientation()
    // Description: The TDx_Sound3DListener::SetOrientation method will set 
    //              the orientation for the listener. 
    //              The front and top vectors meet at right angles within the 
    //              listeners head. 
    //              The front vector may be adjusted to ensure it is at right 
    //              angles to the top vectoor. 
    //              Both vectors may be adjusted to ensure they have a 
    //              magnitude of less than one. 
    //              The front vector indicates the direction of the listeners 
    //              nose, while the top vector points out the top of the 
    //              listeners head. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pXFront - 
    //                The XFront parameter defines a D3DVALUE representing the 
    //                new X coordinate of the front orientation vector. 
    //              pYFront - 
    //                The YFront parameter defines a D3DVALUE representing the 
    //                new Y coordinate of the front orientation vector. 
    //              pZFront - 
    //                The ZFront parameter defines a D3DVALUE representing the 
    //                new Z coordinate of the front orientation vector. 
    //              pXTop - 
    //                The XTop parameter defines a D3DVALUE representing the 
    //                new X coordinate of the top orientation vector. 
    //              pYTop - 
    //                The YTop parameter defines a D3DVALUE representing the 
    //                new Y coordinate of the top orientation vector. 
    //              pZTop - 
    //                The ZTop parameter defines a D3DVALUE representing the 
    //                new Z coordinate of the top orientation vector. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetOrientation( D3DVALUE pXFront, D3DVALUE 
    pYFront, D3DVALUE pZFront, D3DVALUE pXTop, D3DVALUE pYTop, D3DVALUE 
    pZTop, dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetPosition()
    // Description: The TDx_Sound3DListener::SetPosition method will set the 
    //              position, in distance units, for the listener. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pX - 
    //                The X parameter defines a D3DVALUE representing the new X 
    //                coordinate for the position vector. 
    //                This value may be adjusted to prevent floating point 
    //                overflow. 
    //              pY - 
    //                The Y parameter defines a D3DVALUE representing the new Y 
    //                coordinate for the position vector. 
    //                This value may be adjusted to prevent floating point 
    //                overflow. 
    //              pZ - 
    //                The Z parameter defines a D3DVALUE representing the new Z 
    //                coordinate for the position vector. 
    //                This value may be adjusted to prevent floating point 
    //                overflow. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetPosition( D3DVALUE pX, D3DVALUE pY, D3DVALUE 
    pZ, dword pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetRolloffFactor()
    // Description: The TDx_Sound3DListener::SetRolloffFactor method will set 
    //              the rolloff factor for the listener. 
    //              The minimum, maximum and default rolloff values are 
    //              defined in the dsound.h DS3D_MINROLLOFFFACTOR, 
    //              DS3D_MAXROLLOFFFACTOR and DS3D_DEFAULTROLLOFFFACTOR 
    //              members as 0.0, 10.0 and 1.0 respectively. 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pRolloffFactor - 
    //                The RolloffFactor parameter defines the rolloff factor to 
    //                be set for the listener. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetRolloffFactor( D3DVALUE pRolloffFactor, dword 
    pApply ); 

    // ----------------------------------------------------------------------
    // Method:      SetVelocity()
    // Description: The TDx_Sound3DListener::SetVelocity method will set the 
    //              velocity for the listener. 
    //              This value is used to calculate doppler effects only, it 
    //              is up to the application to move the buffer 
    //              appropriately. 
    //              The default velocity is (0,0,0). 
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pX - 
    //                The X parameter defines a D3DVALUE representing the new X 
    //                coordinate of the velocity vector. 
    //              pY - 
    //                The Y parameter defines a D3DVALUE representing the new Y 
    //                coordinate of the velocity vector. 
    //              pZ - 
    //                The Z parameter defines a D3DVALUE representing the new Z 
    //                coordinate of the velocity vector. 
    //              pApply - 
    //                The Apply parameter defines flags indicating whether the 
    //                setting should be applied straight away. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DS3D_DEFERRED -
    //                    Wait until the 
    //                    TDx_Sound3DListener::CommitDeferredSettings method 
    //                    is called before committing the settings. 
    //                    This allows several changes to be made before 
    //                    recalculating. 
    //                  DS3D_IMMEDIATE -
    //                    Apply the settings immediately. 
    //                    The 3D coordinates of all 3D sound buffers will be 
    //                    recalculated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetVelocity( D3DVALUE pX, D3DVALUE pY, D3DVALUE 
    pZ, dword pApply ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_Sound3DListener(TComponent* Owner); 
    virtual __fastcall ~TDx_Sound3DListener(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Sound_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTSOUND3DLISTENER Internal_LPDIRECTSOUND3DLISTENER = { 
    read=FGetInternal_LPDIRECTSOUND3DLISTENER, 
    write=FSetInternal_LPDIRECTSOUND3DLISTENER, nodefault }; 
    __property LPDIRECTSOUND3DLISTENER* Internal_LPDIRECTSOUND3DLISTENER_Ptr 
    = { read=FGetInternal_LPDIRECTSOUND3DLISTENER_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTSOUND3DLISTENER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTSOUND3DLISTENER __fastcall 
    FGetInternal_LPDIRECTSOUND3DLISTENER(); 
    void __fastcall FSetInternal_LPDIRECTSOUND3DLISTENER( 
    LPDIRECTSOUND3DLISTENER pLPDIRECTSOUND3DLISTENER ); 
    LPDIRECTSOUND3DLISTENER* __fastcall 
    FGetInternal_LPDIRECTSOUND3DLISTENER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTSOUND3DLISTENER fLPDIRECTSOUND3DLISTENER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
