// --------------------------------------------------------------------------

// ==========================================================================
// File:         TDx_SoundCapture.CPP
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file contains the code for the TDx_SoundCapture 
//               Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes ...
// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// --------------------------------------------------------------------------
#include "TDx_SoundCapture.H"
// --------------------------------------------------------------------------
// external classes used by TDx_SoundCapture methods.
#include "TDSCBufferDesc.H"
#include "TDx_SoundCaptureBuffer.H"
#include "TDSCCaps.H"
// --------------------------------------------------------------------------
#pragma link "TDx_Library_Defns"
#pragma link "TDx_Library_Functions"
#pragma link "TDx_Sound_Library_Defns"
#pragma link "TDx_Sound_Library_Functions"

// --------------------------------------------------------------------------
// Object Registration...
// --------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
// --------------------------------------------------------------------------
static inline void ValidCtrCheck(TDx_SoundCapture*)
{
    new TDx_SoundCapture(NULL);
}
// --------------------------------------------------------------------------
namespace Tdx_soundcapture
{
    #if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    void __fastcall PACKAGE Register()
    #else
    void __fastcall Register()
    #endif
    {
        TComponentClass classes[1] = {__classid(TDx_SoundCapture)};
        RegisterComponents("TDx_Sound", classes, 0);
    }
}

// --------------------------------------------------------------------------
// Extra code for Callback Handling
// --------------------------------------------------------------------------
TDx_SoundCapture* TDx_SoundCapture_OnDSCaptureEnumerateOwner = NULL;

// --------------------------------------------------------------------------
// Constructor: TDx_SoundCapture::TDx_SoundCapture()
// Description: The default constructor for the TDx_SoundCapture object.
// --------------------------------------------------------------------------
__fastcall TDx_SoundCapture::TDx_SoundCapture(TComponent* Owner)
    : TComponent(Owner)
{
fCreated = false;
fErrorValue = DS_OK;
}
// --------------------------------------------------------------------------
// Destructor:  TDx_SoundCapture::~TDx_SoundCapture()
// Description: The destructor for the TDx_SoundCapture object.
// --------------------------------------------------------------------------
__fastcall TDx_SoundCapture::~TDx_SoundCapture()
{
// destroy internals
if (Created) Destroy();
}
// --------------------------------------------------------------------------
// Property:    Created
// Description: The Created property is true if the internal 
//              LPDIRECTSOUNDCAPTURE used in this component has been 
//              successfully created, otherwise Created is false. 
//              
//              To create the internal LPDIRECTSOUNDCAPTURE, call the 
//              TDx_SoundCapture::Create() method. 
// --------------------------------------------------------------------------
bool __fastcall TDx_SoundCapture::FGetCreated()
{
return fCreated;
}
// --------------------------------------------------------------------------
void __fastcall TDx_SoundCapture::FSetCreated( bool pCreated )
{
fCreated = (pCreated && (fLPDIRECTSOUNDCAPTURE==NULL));
}
// --------------------------------------------------------------------------
// Property:    ErrorValue
// Description: The ErrorValue property contains the last error value 
//              returned from a call to a TDx_SoundCapture method or 
//              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_SoundCapture::FGetErrorValue()
{
return fErrorValue;
}
// --------------------------------------------------------------------------
void __fastcall TDx_SoundCapture::FSetErrorValue( HRESULT pErrorValue )
{
fErrorValue = pErrorValue;
}
// --------------------------------------------------------------------------
// Method:      Create()
// Description: The Create() method is used to automatically create the 
//              internal LPDIRECTSOUNDCAPTURE interface used in the 
//              TDx_SoundCapture component and must be called before any 
//              methods of the TDx_SoundCapture component will function. 
// Params:      pGUID - 
//                The GUID parameter identifies the sound capture device to use 
//                when creating the sound interface. 
//                The TDx_SoundCapture::DSCaptureEnumerate method can be used 
//                to obtain the GUID's of the available sound capture devices. 
// --------------------------------------------------------------------------
bool __fastcall TDx_SoundCapture::Create( GUID* pGUID )
{
// if the component internals have already been created, exit
if (fCreated)
    {
    fErrorValue = TDX_ALREADYCREATED;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ALREADYCREATED", "The "+Name+"::Create() method was called when the component has already been created successfully." );
    return false;
    }

// create internal IDirectSoundCapture interface
fErrorValue = DirectSoundCaptureCreate( pGUID, &fLPDIRECTSOUNDCAPTURE, NULL );

if (fErrorValue!=DS_OK)
    {
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ERROR", "The "+Name+"::Create() method failed because DirectSoundCaptureCreate() failed with error: "+TDx_Sound_Library_ErrorString(fErrorValue)+", "+TDx_Sound_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// successfully created
fCreated = true;
if (FOnCreate) FOnCreate(this);
return true;
}

// --------------------------------------------------------------------------
// Method:      CreateCaptureBuffer()
// Description: The TDx_SoundCapture::CreateCaptureBuffer method will create 
//              a capture buffer. 
//              TDx_SoundCapture cannot extract multiple sounds from one 
//              input sound and only one capture buffer may exist per device. 
//              
//              You can use TDx_SoundCaptureBuffer::Create() method to 
//              achieve the same result as this method with the added 
//              advantage of being able to use the 
//              TDx_SoundCaptureBuffer::OnCreate() event. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DSERR_INVALIDPARAM 
//              DSERR_BADFORMAT 
//              DSERR_GENERIC 
//              DSERR_NODRIVER 
//              DSERR_OUTOFMEMORY 
//              DSERR_UNINITIALIZED 
// Params:      pCBufferDesc - 
//                The CBufferDesc parameter references a TDSCBufferDesc 
//                component holding the properties of the capture buffer to be 
//                created. 
//              pCaptureBuffer - 
//                The CaptureBuffer parameter will reference the new capture 
//                buffer if this method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_SoundCapture::CreateCaptureBuffer( TDSCBufferDesc* pCBufferDesc, TDx_SoundCaptureBuffer* pCaptureBuffer )
{
// Original Function Definition
// HRESULT CreateCaptureBuffer( 
//   LPDSCBUFFERDESC lpDSCBufferDesc,   
//   LPLPDIRECTSOUNDCAPTUREBUFFER lplpDirectSoundCaptureBuffer,   
//   LPUNKNOWN pUnkOuter   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::CreateCaptureBuffer()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pCBufferDesc==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::CreateCaptureBuffer()", "TDX_BADPARAMS", "'pCBufferDesc' cannot be 'NULL'");
    return false;
    }
if (pCaptureBuffer==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::CreateCaptureBuffer()", "TDX_BADPARAMS", "'pCaptureBuffer' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTSOUNDCAPTURE->CreateCaptureBuffer( pCBufferDesc->Internal_DSCBUFFERDESC_Ptr, pCaptureBuffer->Internal_LPDIRECTSOUNDCAPTUREBUFFER_Ptr, NULL );

// Translate Data returned from Function
pCBufferDesc->Internal_DSCBUFFERDESC_Update();
pCaptureBuffer->Internal_LPDIRECTSOUNDCAPTUREBUFFER_Update();

// Handle any Known Results
if (fErrorValue!=DS_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::CreateCaptureBuffer()", TDx_Sound_Library_ErrorString(fErrorValue), TDx_Sound_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      DSCaptureEnumerate()
// Description: The TDx_Sound::DSCaptureEnumerate method will scan all the 
//              sound capture devices installed on the system and call the 
//              TDx_SoundCapture::OnDSCaptureEnumCallback() event for each 
//              device. 
//              
//              There is no need to write a manual callback function, just 
//              place your code in the 
//              TDx_SoundCapture::OnDSCaptureEnumCallback() event. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DSERR_INVALIDPARAM 
// Params:      pContext - 
//                The Context parameter references an application defined 
//                object that is passed to the callback event for each device 
//                enumerated. 
// --------------------------------------------------------------------------
bool __fastcall TDx_SoundCapture::DSCaptureEnumerate( LPVOID pContext )
{
// Original Function Definition
// HRESULT WINAPI DirectSoundCaptureEnumerate( 
//   LPDSENUMCALLBACK lpDSEnumCallback, 
//   LPVOID lpContext );
// 

// Check Parameters for Accuracy

// Call Original Function
TDx_SoundCapture_OnDSCaptureEnumerateOwner = this;
fErrorValue = DirectSoundCaptureEnumerate( Internal_DSCaptureEnumerateCallback, (LPVOID) pContext );
TDx_SoundCapture_OnDSCaptureEnumerateOwner = NULL;

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::DSCaptureEnumerate()", TDx_Sound_Library_ErrorString(fErrorValue), TDx_Sound_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Finished!
return true;
}

// --------------------------------------------------------------------------
// Method:      Destroy()
// Description: The Destroy() method is used to automatically destroy the 
//              internal LPDIRECTSOUNDCAPTURE interface used in the 
//              TDx_SoundCapture component and should be called when the 
//              internal interface is no longer required. 
//              
//              Note: This method is called by the component's destructor. 
// --------------------------------------------------------------------------
bool __fastcall TDx_SoundCapture::Destroy()
{
// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_NOTCREATED", "The "+Name+"::Destroy() method was called before being created successfully." );
    return false;
    }

// code that destroys the internal interface
if (FOnDestroy) FOnDestroy(this);

if (fLPDIRECTSOUNDCAPTURE==NULL) 
    { 
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "The "+Name+" component's internal LPDIRECTSOUNDCAPTURE is NULL but the TDx_Sound component is expecting it to be not NULL. Aborting the Destroy()." );
    return false;
    }

try { 
    fLPDIRECTSOUNDCAPTURE->Release(); 
    } 
catch (...) 
    { 
    fErrorValue = TDX_ERROR; 
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "An exception has occurred within LPDIRECTSOUNDCAPTURE->Release(). Check that you are destroying all components in reverse order of creation." );
    }

// successful destruction
fLPDIRECTSOUNDCAPTURE = NULL;
fCreated = false;
return true;
}

// --------------------------------------------------------------------------
// Method:      GetCaps()
// Description: The TDx_SoundCapture::GetCaps method will return the capture 
//              device capabilities. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DSERR_INVALIDPARAM 
//              DSERR_UNSUPPORTED 
//              DSERR_NODRIVER 
//              DSERR_OUTOFMEMORY 
//              DSERR_UNINITIALIZED 
// Params:      pCCaps - 
//                The CCaps parameter references a TDSCCaps component for 
//                holding the capabilities of the capture device if this method 
//                returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_SoundCapture::GetCaps( TDSCCaps* pCCaps )
{
// Original Function Definition
// HRESULT GetCaps( 
//   LPDSCCAPS lpDSCCaps   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetCaps()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pCCaps==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetCaps()", "TDX_BADPARAMS", "'pCCaps' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTSOUNDCAPTURE->GetCaps( pCCaps->Internal_DSCCAPS_Ptr );

// Translate Data returned from Function
pCCaps->Internal_DSCCAPS_Update();

// Handle any Known Results
if (fErrorValue!=DS_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetCaps()", TDx_Sound_Library_ErrorString(fErrorValue), TDx_Sound_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      Internal_DSCaptureEnumerateCallback()
// Description: The Internal_DSCaptureEnumerateCallback method is used 
//              internally by the TDx_SoundCapture component as a callback 
//              function. 
//              It translates DirectX-style parameters to BCB-style 
//              parameters before calling the 
//              TDx_SoundCapture::OnDSCaptureEnumCallback() event. 
// Params:      pGuid - 
//                This parameter is used internally. 
//              pDescription - 
//                This parameter is used internally. 
//              pModule - 
//                This parameter is used internally. 
//              pContext - 
//                This parameter is used internally. 
// --------------------------------------------------------------------------
BOOL __stdcall TDx_SoundCapture::Internal_DSCaptureEnumerateCallback( LPGUID pGuid, LPCSTR pDescription, LPCSTR pModule, LPVOID pContext )
{
// Original Function Definition
// BOOL WINAPI DSEnumCallback(   
//   LPGUID lpGuid, 
//   LPCSTR lpcstrDescription, 
//   LPCSTR lpcstrModule, 
//   LPVOID lpContext 
// ); 

// Translate Parameters before calling Original Function
GUID* guid = pGuid;
AnsiString driver_description = pDescription;
AnsiString module = pModule;
void* context = pContext;

// transfer to the Callback Event (if it exists)
bool finished = false;
if (TDx_SoundCapture_OnDSCaptureEnumerateOwner->OnDSCaptureEnumerate)
    TDx_SoundCapture_OnDSCaptureEnumerateOwner->OnDSCaptureEnumerate(TDx_SoundCapture_OnDSCaptureEnumerateOwner, guid, driver_description, module, context, finished );

// Finished, return FALSE to abort
if (finished) return FALSE;
return TRUE;
}

// --------------------------------------------------------------------------
// Internal Interface Access
// --------------------------------------------------------------------------
LPDIRECTSOUNDCAPTURE __fastcall TDx_SoundCapture::FGetInternal_LPDIRECTSOUNDCAPTURE()
{
return fLPDIRECTSOUNDCAPTURE;
}
// --------------------------------------------------------------------------
LPDIRECTSOUNDCAPTURE* __fastcall TDx_SoundCapture::FGetInternal_LPDIRECTSOUNDCAPTURE_Ptr()
{
return &fLPDIRECTSOUNDCAPTURE;
}
// --------------------------------------------------------------------------
void __fastcall TDx_SoundCapture::FSetInternal_LPDIRECTSOUNDCAPTURE( LPDIRECTSOUNDCAPTURE pLPDIRECTSOUNDCAPTURE )
{
if (!fCreated)
    {
    fLPDIRECTSOUNDCAPTURE = pLPDIRECTSOUNDCAPTURE;
    fCreated = (fLPDIRECTSOUNDCAPTURE!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_SoundCapture::Internal_LPDIRECTSOUNDCAPTURE_Update()
{
fCreated = (fLPDIRECTSOUNDCAPTURE!=NULL);
}

// --------------------------------------------------------------------------
