#ifndef TDx_SoundCaptureBufferH
#define TDx_SoundCaptureBufferH

// ==========================================================================
// File:         TDx_SoundCaptureBuffer.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDx_SoundCaptureBuffer Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_SoundCaptureBuffer component.
class TDx_SoundCapture;
// --------------------------------------------------------------------------
// external classes used by TDx_SoundCaptureBuffer methods.
class TDSCBufferDesc;
class TDSCBCaps;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_SoundCaptureBuffer
// Description: The TDx_SoundCaptureBuffer component wraps the DirectX 
//              IDirectSoundCaptureBuffer interface which is used in the 
//              manipulation of sound capture buffers. 
//              
//              This component is used to retrieve the capabilities, cursor 
//              positions, format or status of a capture buffer. 
//              
//              The capture of sound data may be stopped and started and the 
//              buffer may be partially or completely locked and unlocked so 
//              the captured sound data may be read. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_SoundCaptureBuffer : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_SoundCaptureBuffer : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Recording
    // Description: The Recording property can be used to determine whether 
    //              or not a capture buffer is currently recording. If true, 
    //              the capture buffer is currently recording. If false, the 
    //              capture buffer is not currently recording. 
    //              
    //              This is equivalent to calling 
    //              TDx_SoundCaptureBuffer::GetStatus() and checking for 
    //              DSCBSTATUS_CAPTURING. 
    // ----------------------------------------------------------------------
    __property bool Recording = { read=FGetRecording, nodefault }; 

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_SoundCaptureBuffer::Create() method after it has 
    //              successfully created the internal 
    //              LPDIRECTSOUNDCAPTUREBUFFER used within the 
    //              TDx_SoundCaptureBuffer component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTSOUNDCAPTUREBUFFER 
    //              interface. 
    //              
    //              For example, you could retrieve the 
    //              TDx_SoundCaptureBuffer component's capabilities, Create() 
    //              dependant components, allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_SoundCaptureBuffer::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_SoundCaptureBuffer::Destroy() method, just before the 
    //              TDx_SoundCaptureBuffer internal 
    //              LPDIRECTSOUNDCAPTUREBUFFER interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_SoundCaptureBuffer::OnCreate() event. For example, 
    //              assume you have already setup the 
    //              TDx_SoundCaptureBuffer::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_SoundCaptureBuffer::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_SoundCaptureBuffer component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTSOUNDCAPTUREBUFFER used in this component has 
    //              been successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTSOUNDCAPTUREBUFFER, call 
    //              the TDx_SoundCaptureBuffer::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_SoundCaptureBuffer method 
    //              or fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    StreamFormat
    // Description: The StreamFormat property references an internal 
    //              WAVEFORMATEX structure defining the format of the current 
    //              stream. 
    //              This property is set during a call to either 
    //              TDx_SoundCaptureBuffer::CreateStream() or 
    //              TDx_SoundCaptureBuffer::SaveToFile() based on the 
    //              user-supplied WAVEFORMATEX structure. 
    // ----------------------------------------------------------------------
    __property WAVEFORMATEX* StreamFormat = { read=FGetStreamFormat, 
    default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    StreamIsOpen
    // Description: The StreamIsOpen property will be set to true if a stream 
    //              is currently open and false otherwise. 
    // ----------------------------------------------------------------------
    __property bool StreamIsOpen = { read=FGetStreamIsOpen, default=false }; 

    // ----------------------------------------------------------------------
    // Property:    StreamSize
    // Description: The StreamSize property holds the total number of bytes 
    //              in the current stream. 
    //              Set this property to the total size of the stream to be 
    //              saved, in bytes, before calling 
    //              TDx_SoundCatureBuffer::SaveToFile(). 
    //              
    //              After calling TDx_SoundCaptureBuffer::CreateFromStream(), 
    //              this property will contain the total number of bytes of 
    //              data read from the stream. 
    //              After calling TDx_SoundCaptureBuffer::CreateStream() or 
    //              TDx_SoundCaptureBuffer::OpenStream() this property will 
    //              be 0. 
    //              After calling TDx_SoundCaptureBuffer::CloseStream() this 
    //              property will contain the total number of bytes read from 
    //              or written to the stream using 
    //              TDx_SoundCaptureBuffer::ReadFromStream() or 
    //              TDx_SoundBuffer::WriteToStream() since the last call to 
    //              TDx_SoundCaptureBuffer::CreateStream() 
    //              After calling TDx_SoundCaptureBuffer::CreateStream() this 
    //              property will contain the total number of bytes read from 
    //              or written to the stream. 
    // ----------------------------------------------------------------------
    __property dword StreamSize = { read=FGetStreamSize, 
    write=FSetStreamSize, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    WaveFileFormat
    // Description: The WaveFileFormat property is used internally to setup 
    //              the .WAV file header for streaming. 
    // ----------------------------------------------------------------------
    __property TDX_WAVE_FILE_FORMAT WaveFileFormat = { 
    read=FGetWaveFileFormat, write=FSetWaveFileFormat, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      CloseStream()
    // Description: The CloseStream method will close the current .wav file. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              TDX_ERROR 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CloseStream(); 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTSOUNDCAPTUREBUFFER interface used in the 
    //              TDx_SoundCaptureBuffer component and must be called 
    //              before any methods of the TDx_SoundCaptureBuffer 
    //              component will function. 
    // Params:      pCaptureBufferDesc - 
    //                The CaptureBufferDesc parameter references a 
    //                TDSBufferDesc defining the properties of the capture 
    //                buffer being created. 
    //              pSoundCapture - 
    //                The SoundCapture parameter references the existing 
    //                TDx_SoundCapture component for which the sound buffer is 
    //                to be created. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDSCBufferDesc* pCaptureBufferDesc, 
    TDx_SoundCapture* pSoundCapture ); 

    // ----------------------------------------------------------------------
    // Method:      CreateStream()
    // Description: The CreateStream method will create a new .wav file using 
    //              the format of the WAVEFORMATEX structure supplied. 
    //              
    //              If the file already exists, it is replaced. 
    //              If the file does not exist, it will be created if 
    //              possible. 
    //              If the file cannot be created, the file remnants will 
    //              remain. 
    //              
    //              If the method call is successful, the method returns 
    //              true, and the TDx_SoundCaptureBuffer::StreamIsOpen 
    //              property is true. 
    //              When open for streaming, raw sound data to the stream 
    //              using the TDx_SoundCaptureBuffer::WriteToStream() method. 
    //              When finished streaming, you can close the stream using 
    //              the TDx_SoundCaptureBuffer::CloseStream() method. 
    //              
    //              If the method call fails, the method returns false, and 
    //              the OnError event will be triggered with one of the 
    //              following values: 
    //              
    //                TDX_BADPARAMS 
    //                TDX_ERROR 
    // Params:      pFileName - 
    //                The FileName parameter defines the path and filename of 
    //                the  .wav file to be created. 
    //              pFormat - 
    //                The Format method parameter describes the format of the 
    //                sound file to be created. This parameter must point to a 
    //                user-supplied WAVEFORMATEX structure that has been filled 
    //                with appropriate values. 
    //                
    //                You can retrieve the WAVEFORMATEX of the current buffer 
    //                using the TDx_SoundCaptureBuffer::GetFormat()  method. 
    //                
    //                You can also pass in a non-WAVE_FORMAT_PCM structure 
    //                typecast as a WAVEFORMATEX, and supply the byte-size of 
    //                the non-WAVE_FORMAT_PCM using the FormatSize parameter. 
    //              pFormatSize - 
    //                The FormatSize parameter must contain the size of the 
    //                WAVEFORMATEX structure passed into this method via the 
    //                Format parameter. Generally this will be 
    //                sizeof(WAVEFORMATEX) but it may differ depending on the 
    //                definition of the WAVEFORMATEX.. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CreateStream( AnsiString pFileName, WAVEFORMATEX* 
    pFormat, int pFormatSize ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTSOUNDCAPTUREBUFFER interface used in the 
    //              TDx_SoundCaptureBuffer component and should be called 
    //              when the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetCaps()
    // Description: The TDx_SoundCaptureBuffer::GetCaps method will return 
    //              the capture buffer's capabilities. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_UNSUPPORTED 
    //              DSERR_OUTOFMEMORY 
    // Params:      pCBCaps - 
    //                The CBCaps parameter references a TDSCBCaps component for 
    //                holding the capture buffer capabilities if this method 
    //                returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetCaps( TDSCBCaps* pCBCaps ); 

    // ----------------------------------------------------------------------
    // Method:      GetCurrentPosition()
    // Description: The TDx_SoundCaptureBuffer::GetCurrentPosition method 
    //              will return the current position of the capture and read 
    //              pointers within the buffer. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_NODRIVER 
    //              DSERR_OUTOFMEMORY 
    // Params:      pCapturePosition - 
    //                The CapturePosition parameter will reference the offset 
    //                of the capture cursor within the capture buffer if this 
    //                method returns successfully. 
    //                
    //                Set this parameter to NULL if the capture cursor position 
    //                is not required. 
    //              pReadPosition - 
    //                The ReadPosition parameter will reference the offset of 
    //                the read cursor within the capture buffer if this method 
    //                returns successfully. 
    //                
    //                Set this parameter to NULL if the read cursor position is 
    //                not required. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetCurrentPosition( dword* pCapturePosition, 
    dword* pReadPosition ); 

    // ----------------------------------------------------------------------
    // Method:      GetFormat()
    // Description: The TDx_SoundCaptureBuffer::GetFormat method will return 
    //              the sound format currently set for the capture buffer. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pFormat - 
    //                The Format parameter references a WAVEFORMATEX structure 
    //                for holding the sound data format of the capture buffer 
    //                if this method returns successfully. 
    //                
    //                Setting this parameter to NULL will cause the buffer size 
    //                needed to be retrieved and placed in the SizeWritten 
    //                parameter of this method. 
    //              pSizeAllocated - 
    //                The SizeAllocated parameter defines the size, in bytes, 
    //                of the WAVEFORMATEX structure referenced by the Format 
    //                parameter of this method. 
    //                If the structure exceeds this value it will be truncated. 
    //              pSizeWritten - 
    //                The SizeWritten parameter will reference the amount, in 
    //                bytes, that was written into the WAVEFORMATEX structure 
    //                referenced by the Format parameter if this method returns 
    //                successfully. 
    //                
    //                This parameter may be set to NULL. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetFormat( WAVEFORMATEX* pFormat, dword 
    pSizeAllocated, dword* pSizeWritten ); 

    // ----------------------------------------------------------------------
    // Method:      GetStatus()
    // Description: The TDx_SoundCaptureBuffer::GetStatus method will return 
    //              the current status of the capture buffer. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              DSERR_INVALIDPARAM 
    // Params:      pStatus - 
    //                The Status parameter references flags indicating the 
    //                current status of the capture buffer. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DSBSTATUS_LOOPING -
    //                    The buffer is looping. 
    //                    The buffer returns to the beginning and continues 
    //                    capturing when it reaches the end of the buffer. 
    //                    When this flag is set, the buffer will also be 
    //                    capturing. 
    //                  DSCBSTATUS_CAPTURING -
    //                    The buffer is currently capturing. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetStatus( dword* pStatus ); 

    // ----------------------------------------------------------------------
    // Method:      Lock()
    // Description: The TDx_SoundCaptureBuffer::Lock method will return 
    //              pointers into the capture buffer, allowing the reading or 
    //              writing of audio data. 
    //              
    //              The capture buffer should not be locked for long periods 
    //              as the capture cursor will reach the locked bytes cause 
    //              configuration dependent audio problems. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_INVALIDCALL 
    // Params:      pReadCursor - 
    //                The ReadCursor parameter defines the offset, in bytes, to 
    //                where the lock is to begin in the buffer. 
    //              pReadBytes - 
    //                The ReadBytes parameter defines the amount, in bytes, of 
    //                the buffer that is to be locked. 
    //                
    //                The space that is locked may wrap around to the start of 
    //                the buffer, which is conceptually circular, in which case 
    //                the AudioPtr2 parameter of this method will reference the 
    //                start of the buffer. 
    //                
    //                Setting DSCBLOCK_ENTIREBUFFER in the Flags parameter of 
    //                this method will cause this parameter to be ignored. 
    //              pAudioPtr1 - 
    //                The AudioPtr1 parameter will reference the first block of 
    //                locked capture buffer if this method returns 
    //                successfully. 
    //              pAudioBytes1 - 
    //                The AudioBytes1 parameter references the number of bytes 
    //                referenced by the AudioPtr1 parameter of this method. 
    //                
    //                When this value is less than the value in the ReadBytes 
    //                parameter of this method, it means AudioPtr2 will 
    //                reference a second block of sound data. 
    //              pAudioPtr2 - 
    //                The AudioPtr2 parameter will reference the second block 
    //                of locked capture buffer if this method returns 
    //                successfully. 
    //                
    //                When this parameter is NULL it means AudioPtr1 references 
    //                the entire locked region of the sound buffer and any 
    //                portion of the area that wraps around will not be locked. 
    //              pAudioBytes2 - 
    //                The AudioBytes2 parameter references the number of bytes 
    //                referenced by the AudioPtr2 parameter of this method. 
    //                
    //                When the AudioPtr2 parameter of this method is set to 
    //                NULL, this parameter will be 0. 
    //              pFlags - 
    //                The Flags parameter defines a flag modifying the lock 
    //                event. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DSBLOCK_ENTIREBUFFER -
    //                    Lock the whole capture buffer. 
    //                    Setting this flag causes the ReadBytes parameter of 
    //                    this method to be ignored. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Lock( dword pReadCursor, dword pReadBytes, void** 
    pAudioPtr1, dword* pAudioBytes1, void** pAudioPtr2, dword* pAudioBytes2, 
    dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      SaveToFile()
    // Description: The SaveToFile ()method will save the entire sound buffer 
    //              to a specified .WAV file. 
    //              Calling this method is equivalent to calling the 
    //              following methods in the order shown here :- 
    //              
    //                TDx_SoundCaptureBuffer::CreateStream() 
    //                TDx_SoundCaptureBuffer::Lock() 
    //                TDx_SoundCaptureBuffer::WriteToStream() 
    //                TDx_SoundCaptureBuffer::Unlock() 
    //                TDx_SoundCaptureBuffer::CloseStream() 
    //              
    //              If the file already exists, it is replaced. 
    //              If the file does not exist, it will be created if 
    //              possible. 
    //              If the file cannot be created, the file remnants will 
    //              remain. 
    //              
    //              If you wish to save multiple chunks of sound data,  do 
    //              this :- 
    //              
    //                TDx_SoundCaptureBuffer::CreateStream() 
    //                TDx_SoundCaptureBuffer::Lock() 
    //                TDx_SoundCaptureBuffer::WriteToStream() 
    //                TDx_SoundCaptureBuffer::WriteToStream() 
    //                TDx_SoundCaptureBuffer::WriteToStream() 
    //                TDx_SoundCaptureBuffer::WriteToStream() 
    //                TDx_SoundCaptureBuffer::Unlock() 
    //                TDx_SoundCaptureBuffer::CloseStream() 
    //              
    //              Before calling this method, you must setup the 
    //              TDx_SoundCaptureBuffer::StreamFormat property, 
    //              and also the TDx_SoundCaptureBuffer::StreamSize property. 
    //              
    //              After this method returns, the 
    //              TDx_SoundCaptureBuffer::StreamSize property contains the 
    //              total number of bytes streamed. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              
    //                TDX_BADPARAMS 
    //                TDX_ERROR 
    // Params:      pFileName - 
    //                The FileName parameter defines the path and filename of 
    //                the new .wav file which is to be created. 
    //              pFormat - 
    //                The Format method parameter describes the format of the 
    //                sound file to be created. 
    //                This parameter must point to a user-supplied WAVEFORMATEX 
    //                structure that has been filled with appropriate values. 
    //              pFormatSize - 
    //                The FormatSize parameter must contain the size of the 
    //                WAVEFORMATEX structure passed into this method via the 
    //                Format parameter. 
    //                Generally this will be sizeof(WAVEFORMATEX) but it may 
    //                differ depending on the definition of the WAVEFORMATEX. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SaveToFile( AnsiString pFileName, WAVEFORMATEX* 
    pFormat, int pFormatSize ); 

    // ----------------------------------------------------------------------
    // Method:      Start()
    // Description: The TDx_SoundCaptureBuffer::Start method will start 
    //              retrieving sound data into this capture buffer. 
    //              
    //              This method will have no effect if the buffer is already 
    //              capturing. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_NODRIVER 
    //              DSERR_OUTOFMEMORY 
    // Params:      pFlags - 
    //                The Flags parameter defines a flag specifying capture 
    //                buffer behavior when capturing sound data. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DSCBSTART_LOOPING -
    //                    The capture buffer is looping. 
    //                    The buffer returns to the beginning and continues 
    //                    capturing when it reaches the end of the buffer. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Start( dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      Stop()
    // Description: The TDx_SoundCaptureBuffer::Stop method will halt the 
    //              capture of data into this buffer. 
    //              This method will have no effect if the buffer is already 
    //              stopped. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_NODRIVER 
    //              DSERR_OUTOFMEMORY 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Stop(); 

    // ----------------------------------------------------------------------
    // Method:      Unlock()
    // Description: The TDx_SoundCaptureBuffer::Unlock method will unlock a 
    //              capture buffer previously locked with 
    //              TDx_SoundCaptureBuffer::Lock(). 
    //              
    //              Both pointers and both byte values returned by 
    //              TDx_SoundCaptureBuffer::Lock() must be supplied, even if 
    //              0 bytes were locked with the second pointer. 
    //              
    //              The capture buffer should not be locked for long periods 
    //              as the capture cursor may reach the locked bytes and 
    //              possibly cause configuration dependent audio problems. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_INVALIDCALL 
    // Params:      pAudioPtr1 - 
    //                The AudioPtr1 parameter should reference the value 
    //                returned in the AudioPtr1 parameter of 
    //                TDx_SoundCaptureBuffer::Lock(). 
    //              pAudioBytes1 - 
    //                The AudioBytes1 parameter defines the number of bytes 
    //                read from the area referenced by the AudioPtr1 parameter. 
    //              pAudioPtr2 - 
    //                The AudioPtr2 parameter should reference the value 
    //                returned in the AudioPtr2 parameter of 
    //                TDx_SoundCaptureBuffer::Lock(). 
    //              pAudioBytes2 - 
    //                The AudioBytes2 parameter defines the number of bytes 
    //                read from the area referenced by the AudioPtr2 parameter. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Unlock( void* pAudioPtr1, dword pAudioBytes1, 
    void* pAudioPtr2, dword pAudioBytes2 ); 

    // ----------------------------------------------------------------------
    // Method:      WriteToStream()
    // Description: The WriteToStream method will write a specified amount of 
    //              sound data to the current .wav file. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              TDX_BADPARAMS 
    //              TDX_ERROR 
    // Params:      pBytesToWrite - 
    //                The BytesToWrite parameter defines the amount, in bytes, 
    //                of sound data to write to the current .wav file. 
    //              pSoundData - 
    //                The SoundData parameter references the array of bytes 
    //                that will be written to the current .wav file. 
    //                
    //                This parameter may be set to one of the pointers returned 
    //                by TDx_SoundCaptureBuffer::Lock() to write sound data 
    //                directly from the capture buffer. 
    //              pBytesActuallyWritten - 
    //                The BytesActuallyWritten parameter will reference the 
    //                amount, in bytes, that was actually written to the file 
    //                if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall WriteToStream( dword pBytesToWrite, byte* 
    pSoundData, dword* pBytesActuallyWritten ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_SoundCaptureBuffer(TComponent* Owner); 
    virtual __fastcall ~TDx_SoundCaptureBuffer(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Sound_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTSOUNDCAPTUREBUFFER Internal_LPDIRECTSOUNDCAPTUREBUFFER 
    = { read=FGetInternal_LPDIRECTSOUNDCAPTUREBUFFER, 
    write=FSetInternal_LPDIRECTSOUNDCAPTUREBUFFER, nodefault }; 
    __property LPDIRECTSOUNDCAPTUREBUFFER* 
    Internal_LPDIRECTSOUNDCAPTUREBUFFER_Ptr = { 
    read=FGetInternal_LPDIRECTSOUNDCAPTUREBUFFER_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTSOUNDCAPTUREBUFFER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    StreamHandle
    // Description: The StreamHandle property is used internally by the 
    //              TDx_SoundCaptureBuffer component to implement sound 
    //              streaming. 
    // ----------------------------------------------------------------------
    __property int StreamHandle = { read=FGetStreamHandle, 
    write=FSetStreamHandle, default=-1 }; 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    bool __fastcall FGetRecording(); 
    WAVEFORMATEX* __fastcall FGetStreamFormat(); 
    int __fastcall FGetStreamHandle(); 
    void __fastcall FSetStreamHandle( int pStreamHandle ); 
    bool __fastcall FGetStreamIsOpen(); 
    dword __fastcall FGetStreamSize(); 
    void __fastcall FSetStreamSize( dword pStreamSize ); 
    TDX_WAVE_FILE_FORMAT __fastcall FGetWaveFileFormat(); 
    void __fastcall FSetWaveFileFormat( TDX_WAVE_FILE_FORMAT pWaveFileFormat 
    ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTSOUNDCAPTUREBUFFER __fastcall 
    FGetInternal_LPDIRECTSOUNDCAPTUREBUFFER(); 
    void __fastcall FSetInternal_LPDIRECTSOUNDCAPTUREBUFFER( 
    LPDIRECTSOUNDCAPTUREBUFFER pLPDIRECTSOUNDCAPTUREBUFFER ); 
    LPDIRECTSOUNDCAPTUREBUFFER* __fastcall 
    FGetInternal_LPDIRECTSOUNDCAPTUREBUFFER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 
    dword fRecording; 
    WAVEFORMATEX* fStreamFormat; 
    int fStreamHandle; 
    bool fStreamIsOpen; 
    dword fStreamSize; 
    TDX_WAVE_FILE_FORMAT fWaveFileFormat; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTSOUNDCAPTUREBUFFER fLPDIRECTSOUNDCAPTUREBUFFER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
