#ifndef TDx_SoundKsPropertySetH
#define TDx_SoundKsPropertySetH

// ==========================================================================
// File:         TDx_SoundKsPropertySet.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDx_SoundKsPropertySet Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_SoundKsPropertySet component.
class TDx_SoundBuffer;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_SoundKsPropertySet
// Description: The TDx_SoundKsPropertySet component wraps the DirectX 
//              IKsPropertySet interface which is used to retrieve and 
//              utilize extended properties of sound devices. 
//              
//              This component is used to determine if support for 
//              non-standard properties of the sound device is available and 
//              also to set or retrieve those properties. 
//              
//              It accesses the properties of the sound driver, thus allowing 
//              access to any additional features that are unknown to the 
//              main body of the TDx_Sound_Library. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_SoundKsPropertySet : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_SoundKsPropertySet : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_SoundKsPropertySet::Create() method after it has 
    //              successfully created the internal LPKSPROPERTYSET used 
    //              within the TDx_SoundKsPropertySet component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPKSPROPERTYSET interface. 
    //              
    //              For example, you could retrieve the 
    //              TDx_SoundKsPropertySet component's capabilities, Create() 
    //              dependant components, allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_SoundKsPropertySet::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_SoundKsPropertySet::Destroy() method, just before the 
    //              TDx_SoundKsPropertySet internal LPKSPROPERTYSET interface 
    //              is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_SoundKsPropertySet::OnCreate() event. For example, 
    //              assume you have already setup the 
    //              TDx_SoundKsPropertySet::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_SoundKsPropertySet::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_SoundKsPropertySet component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPKSPROPERTYSET used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPKSPROPERTYSET, call the 
    //              TDx_SoundKsPropertySet::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_SoundKsPropertySet method 
    //              or fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPKSPROPERTYSET interface used in the 
    //              TDx_SoundKsPropertySet component and must be called 
    //              before any methods of the TDx_SoundKsPropertySet 
    //              component will function. 
    // Params:      pSoundBuffer - 
    //                The SoundBuffer parameter references the existing 
    //                TDx_SoundBuffer component for which the sound buffer is 
    //                to be created. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_SoundBuffer* pSoundBuffer ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPKSPROPERTYSET interface used in the 
    //              TDx_SoundKsPropertySet component and should be called 
    //              when the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      Get()
    // Description: The TDx_SoundKsPropertySet::Get method will return the 
    //              value currently set for a property. 
    //              
    //              The value of the property will be copied to the buffer 
    //              referenced by the PropertyData parameter of this method. 
    //              
    //              The format of the data in the InstanceData and Property 
    //              parameters is item specific. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              E_POINTER 
    // Params:      pPropertySet - 
    //                The PropertySet parameter defines the GUID of the 
    //                property set to be accessed. 
    //              pId - 
    //                The Id parameter defines the identifier of the individual 
    //                property to be retrieved. 
    //                This must be a property contained within the property set 
    //                indicated by the PropertySetId parameter of this method. 
    //                This identifier is indexed from zero and will always be 
    //                consistant for a given property set GUID. 
    //              pInstanceData - 
    //                The InstanceData parameter references the instance data 
    //                for the retrieval. 
    //              pInstanceLength - 
    //                The InstanceLength parameter number of bytes referenced 
    //                by the InstanceData parameter of this method. 
    //              pPropertyData - 
    //                The PropertyData parameter references the property data 
    //                buffer for the retrieval. 
    //              pDataLength - 
    //                The DataLength parameter defines number of bytes 
    //                referenced by the PropertyData parameter of this method. 
    //              pBytesReturned - 
    //                The BytesReturned parameter defines the number of bytes 
    //                written to the PropertyData parameter of this method. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Get( REFGUID pPropertySet, ulong pId, void* 
    pInstanceData, ulong pInstanceLength, void* pPropertyData, ulong 
    pDataLength, ulong* pBytesReturned ); 

    // ----------------------------------------------------------------------
    // Method:      QuerySupport()
    // Description: The TDx_SoundKsPropertySet::QuerySupport method will 
    //              determine whether a component supports a specific 
    //              property set. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              E_NOTIMPL 
    //              E_POINTER 
    // Params:      pPropertySetId - 
    //                The PropertySetId parameter defines the GUID of the 
    //                property set. 
    //              pPropertyId - 
    //                The PropertyId parameter defines the identifier of the 
    //                property within the property set to check for support. 
    //                This identifier is indexed from zero and will always be 
    //                consistant for a given property set GUID. 
    //              pSupport - 
    //                The Support parameter will reference flags indicating the 
    //                support available for the specified property if this 
    //                method returns successfully. 
    //                The described effect applies when the flag is set. 
    //                This parameter will be set to zero if the property or 
    //                property set does not exist. 
    //                Flags:
    //                  KSPROPERTY_SUPPORT_GET -
    //                    The property can be read using 
    //                    TDx_SoundKsPropertySet::Get. 
    //                  KSPROPERTY_SUPPORT_SET -
    //                    The property can be changed using 
    //                    TDx_SoundKsPropertySet::Set. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall QuerySupport( REFGUID pPropertySetId, ulong 
    pPropertyId, ulong* pSupport ); 

    // ----------------------------------------------------------------------
    // Method:      Set()
    // Description: The TDx_SoundKsPropertySet::Set method will set a 
    //              property to the specified value. 
    //              The format of the data in the InstanceData and Property 
    //              parameters is item specific. 
    //              
    //              If the method call fails, the OnError event may be 
    //              triggered with the following value: 
    //              E_POINTER 
    // Params:      pPropertySet - 
    //                The PropertySet parameter defines the GUID of the 
    //                property set. 
    //              pId - 
    //                The Id parameter defines the identifier of the individual 
    //                property to be set. 
    //                This must be a property contained within the property set 
    //                indicated by the PropertySetId parameter of this method. 
    //                This identifier is indexed from zero and will always be 
    //                consistant for a given property set GUID. 
    //              pInstanceData - 
    //                The InstanceData parameter references the which instance 
    //                is to be set. 
    //                This parameter is used to uniquely identify an object 
    //                when there are multiple objects within the port this 
    //                operation is acting on. 
    //              pInstanceLength - 
    //                The InstanceLength parameter defines number of bytes 
    //                referenced by the InstanceData parameter of this method. 
    //              pPropertyData - 
    //                The PropertyData parameter references a buffer holding 
    //                the information to be set for the specified property. 
    //              pDataLength - 
    //                The DataLength parameter defines the number of bytes 
    //                referenced by the PropertyData parameter of this method. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Set( REFGUID pPropertySet, ulong pId, void* 
    pInstanceData, ulong pInstanceLength, void* pPropertyData, ulong 
    pDataLength ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_SoundKsPropertySet(TComponent* Owner); 
    virtual __fastcall ~TDx_SoundKsPropertySet(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Sound_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPKSPROPERTYSET Internal_LPKSPROPERTYSET = { 
    read=FGetInternal_LPKSPROPERTYSET, write=FSetInternal_LPKSPROPERTYSET, 
    nodefault }; 
    __property LPKSPROPERTYSET* Internal_LPKSPROPERTYSET_Ptr = { 
    read=FGetInternal_LPKSPROPERTYSET_Ptr, nodefault }; 
    void __fastcall Internal_LPKSPROPERTYSET_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPKSPROPERTYSET __fastcall FGetInternal_LPKSPROPERTYSET(); 
    void __fastcall FSetInternal_LPKSPROPERTYSET( LPKSPROPERTYSET 
    pLPKSPROPERTYSET ); 
    LPKSPROPERTYSET* __fastcall FGetInternal_LPKSPROPERTYSET_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPKSPROPERTYSET fLPKSPROPERTYSET; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
