#ifndef TDx_SoundNotifyH
#define TDx_SoundNotifyH

// ==========================================================================
// File:         TDx_SoundNotify.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDx_SoundNotify Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_SoundNotify component.
class TDx_SoundBuffer;
class TDx_SoundCaptureBuffer;
// --------------------------------------------------------------------------
// external classes used by TDx_SoundNotify properties.
class TDSBPositionNotify;

// --------------------------------------------------------------------------
// Definition for the OnNotify event.
typedef __fastcall void (__closure*     TDx_NotifyEvent) 
                        ( TObject*      Sender, 
                          int           pNotification ); 

// ==========================================================================
// Class:       TDx_SoundNotify
// Description: The TDx_SoundNotify component wraps the DirectX 
//              IDirectSoundNotify interface which allows notification events 
//              to be set up for playback or capture buffers. 
//              
//              This component is used to define positions within a sound or 
//              capture buffer that will trigger a 
//              TDx_SoundNotify::OnNotify() event when the play or capture 
//              cursor reaches them. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_SoundNotify : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_SoundNotify : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    LocalNotify
    // Description: The LocalNotify property is used to toggle the 
    //              destination for DirectX buffer notification events 
    //              created using 
    //              TDx_SoundNotify::SetNotificationPositions(). 
    //              
    //              If the LocalNotify property is true, the 
    //              TDx_SoundNotify::OnNotify() event is called with 
    //              (0..number_of_events-1). 
    //              
    //              If the LocalNotify property is false, the 
    //              TDSBPositionNotify::OnNotify() event of the appropriate 
    //              component is called. 
    //              
    //              By default, LocalNotify is true. 
    // ----------------------------------------------------------------------
    __property bool LocalNotify = { read=FGetLocalNotify, 
    write=FSetLocalNotify, default=true }; 

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_SoundNotify::Create() method after it has 
    //              successfully created the internal LPDIRECTSOUNDNOTIFY 
    //              used within the TDx_SoundNotify component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTSOUNDNOTIFY interface. 
    //              
    //              For example, you could retrieve the TDx_SoundNotify 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_SoundNotify::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_SoundNotify::Destroy() method, just before the 
    //              TDx_SoundNotify internal LPDIRECTSOUNDNOTIFY interface is 
    //              destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_SoundNotify::OnCreate() 
    //              event. For example, assume you have already setup the 
    //              TDx_SoundNotify::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_SoundNotify::OnDestroy() event to deallocate memory, 
    //              Destroy() the dependant components and, when required, 
    //              destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_SoundNotify component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnNotify()
    // Description: The OnNotify event is triggered when the buffer reaches a 
    //              point previously defined using 
    //              TDx_SoundNotify::SetNotificationPositions(). 
    //              
    //              This is useful for loading or writing the next chunk of 
    //              data, cascading sound effects and so on. 
    //              
    //              Set the LocalNotify property of this component to false 
    //              if you wish to use the TDSBPositionNotify::OnNotify() 
    //              event corresponding to the triggering notification 
    //              position or true to use this OnNotify event. 
    //              
    //              When using this event, the Value parameter indicates 
    //              which notification position has been reached. 
    // ----------------------------------------------------------------------
    __property TDx_NotifyEvent OnNotify = {read=FOnNotify, write=FOnNotify, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTSOUNDNOTIFY used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTSOUNDNOTIFY, call the 
    //              TDx_SoundNotify::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_SoundNotify method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    NumPositionNotifies
    // Description: The NumPositionNotifies property holds the number of 
    //              notification positions set during the last call to 
    //              TDx_SoundNotify::SetNotificationPositions(). 
    //              
    //              This property is used internally but can be examined for 
    //              external use. 
    // ----------------------------------------------------------------------
    __property dword NumPositionNotifies = { read=FGetNumPositionNotifies, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PositionNotifies
    // Description: The PositionNotifies property is used internally to keep 
    //              track of the TDSBPositionNotifies component most recently 
    //              used by the TDx_SoundNotify::SetNotificationPositions() 
    //              method; 
    //              
    //              This property is used internally but can be examined for 
    //              external use. 
    // ----------------------------------------------------------------------
    __property TDSBPositionNotify* PositionNotifies = { 
    read=FGetPositionNotifies, default=NULL }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTSOUNDNOTIFY interface used in the 
    //              TDx_SoundNotify component and must be called before any 
    //              methods of the TDx_SoundNotify component will function. 
    // Params:      pSoundBuffer - 
    //                The SoundBuffer parameter references the existing 
    //                TDx_SoundBuffer or TDx_SoundCaptureBuffer component for 
    //                which the TDx_SoundNotify component is to be created. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TComponent* pSoundBuffer ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTSOUNDNOTIFY interface used in the 
    //              TDx_SoundNotify component and should be called when the 
    //              internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      SetNotificationPositions()
    // Description: The TDx_SoundNotify::SetNotificationPositions method will 
    //              set the notification positions for triggering events when 
    //              the play or read cursor reaches them. Before calling this 
    //              method, make sure to setup the TDSBPositionNotify 
    //              multiwrapper component with appropriate data. The 
    //              positions specified in this call will supercede any 
    //              previously set positions. The buffer must be stopped for 
    //              this method to work. 
    //              
    //              Note: DirectSound seems to limit the number of 
    //              notification positions that can be set on any individual 
    //              buffer to 60. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_OUTOFMEMORY 
    // Params:      pNumPositionNotifies - 
    //                The NumPositionNotifies parameter defines the number of 
    //                TDSBPositionNotify components referenced by the 
    //                NumPositionNotifies parameter of this method. 
    //              pPositionNotifies - 
    //                The PositionNotifies parameter references a 
    //                TDSBPositionNotify multiwrapper component which contains 
    //                the notification positions offsets to be set for the 
    //                buffer. Note: Do not set the TDSBPositionNotify::Event 
    //                property when using this method. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetNotificationPositions( dword 
    pNumPositionNotifies, TDSBPositionNotify* pPositionNotifies ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_SoundNotify(TComponent* Owner); 
    virtual __fastcall ~TDx_SoundNotify(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Sound_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTSOUNDNOTIFY Internal_LPDIRECTSOUNDNOTIFY = { 
    read=FGetInternal_LPDIRECTSOUNDNOTIFY, 
    write=FSetInternal_LPDIRECTSOUNDNOTIFY, nodefault }; 
    __property LPDIRECTSOUNDNOTIFY* Internal_LPDIRECTSOUNDNOTIFY_Ptr = { 
    read=FGetInternal_LPDIRECTSOUNDNOTIFY_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTSOUNDNOTIFY_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    bool __fastcall FGetLocalNotify(); 
    void __fastcall FSetLocalNotify( bool pLocalNotify ); 
    dword __fastcall FGetNumPositionNotifies(); 
    TDSBPositionNotify* __fastcall FGetPositionNotifies(); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTSOUNDNOTIFY __fastcall FGetInternal_LPDIRECTSOUNDNOTIFY(); 
    void __fastcall FSetInternal_LPDIRECTSOUNDNOTIFY( LPDIRECTSOUNDNOTIFY 
    pLPDIRECTSOUNDNOTIFY ); 
    LPDIRECTSOUNDNOTIFY* __fastcall FGetInternal_LPDIRECTSOUNDNOTIFY_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 
    bool fLocalNotify; 
    dword fNumPositionNotifies; 
    TDSBPositionNotify* fPositionNotifies; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTSOUNDNOTIFY fLPDIRECTSOUNDNOTIFY; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;
    TDx_NotifyEvent FOnNotify;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
